@@ -49,7 +49,9 @@ weave ANY:aspect CMETHODN:method @stmt @token @new
 aspect TYPEN:type cmethod.. CLASSN:class METHODN:method STT_DECL:params {:instmt  @stmt @token @new
 
 if pexpr:bool {:truecase else.. {:falsecase  @stmt @token @new @name @cmpl
-else @token 
+else @token
+switch pexpr:bool {:case @token @stmt @new @name @cmpl
+case @token 
 while pexpr:bool {:loop @token @stmt @new @name @cmpl
 do {:loop pexpr:bool ; @stmt @token @new @name @cmpl
 for pstmt3.. stmt:init expr:bool stmt:redo {:loop  @stmt @token @new @name @cmpl
@@ -61,10 +63,10 @@ break    any_:label ; @stmt @token @new @cmpl
 continue any_:label ;  @stmt @token @new @cmpl
 goto ANY:label ;  @stmt @token @new @cmpl
 
-try {:trycase STT_CATCH:catch {:finally   @stmt  @token @new
+try {:trycase STT_CATCH:catch {:finally   @stmt  @token @new  @name @cmpl
 catch EXPTN:type VARN:name {:catch   @token @stmt
 finally @token
-throw expr:expt ;   @stmt  @token @new
+throw expr:expt ;   @stmt  @token @new @name @cmpl
 err ANY:msg     @stmt @cmpl
 
 let      VARN:lvalue expr:rvalue     @stmt  @name @cmpl  @expr
@@ -90,19 +92,19 @@ letmulti VARN*:names[]               @stmt
 print  expr*:values[]  @stmt @token @new  @cmpl
 assert pexpr:bool ;    @token @stmt @new 
 #utest  pexpr:bool {:testcase  @stmt @test
-man   ANY:keyword ;  @stmt @new
+#man   ANY:keyword ;  @stmt @new
 
 '''
 
 TOKEN = '''
-{ BRACE          @array     @top
-( PARENTHESIS    @array     @top
-[ BRANCET        @array     @top
+{ BRACE          @array     @top @P99
+( PARENTHESIS    @array     @top @P99
+[ BRANCET        @array     @top @P99
 ... DOTS         @top
 
 ; SEMICOLON      @stmt     @eos
-, COMMA          
-= LET            
+, COMMA          @P0
+= LET            @P0
 <<= lshifte      
 >>= rshifte       
 += adde           
@@ -113,77 +115,79 @@ TOKEN = '''
 |= bitore        
 &= bitande       
 ?\?= ALTLET      
-? QUESTION       @top
-.. SUBSETE        
-..< SUBSET        
-..+ OFFSET       
-: COLON          
+? QUESTION       @P2
+.. SUBSETE       @P2
+..< SUBSET       @P2
+..+ OFFSET       @P2
+: COLON          @P2
+?\? ALT          @op    @P2
 
-|| OR
-or TT_OR         @alias            
-&& AND
-and TT_AND       @alias
-!  Not           @top  @op
-not TT_NOT       @alias
+|| OR            @P5
+or TT_OR         @alias @P5
+&& AND           @P6
+and TT_AND       @alias  @P6
+!  Not           @top  @op  @P7 @A1
+not TT_NOT       @alias  @P7  @A1
 
-instanceof Instanceof  @op
-isa? Isa                @op
-isa  TT_ISA @alias
-as  As                 @op
-in? Has                 @op
-#in  TT_IN             @alias
-=== Is                 @op @2
-is?  TT_IS @alias
-to? To                 @op @2
--->  TT_TO @alias
-== Eq                  @op @2
-!= Neq                 @op @2
-< Lt                   @op @2
-<= Lte                 @op @2
-> Gt                   @op @2
->= Gte                 @op @2
+instanceof Instanceof  @op    @P9  @A2
+isa? Isa               @op    @P9  @A2
+isa  TT_ISA @alias     @P9  @A2
+as  As                 @op  @P9  @A2
+in? Has                @op  @P9  @A2
 
-| Lor            @op @top
-%% Fmt                 @op @2
+=== Is                 @op @A2  @P9
+is?  TT_IS @alias      @op @A2  @P9
+to? To                 @op @A2  @P9
+-->  TT_TO @alias      @op @A2  @P9
 
-<< Lshift              @op @2
->> Rshift              @op @2
-+ Add                  @op @top
-- Sub                  @op @top
-/ Div                  @op @2
-% Mod                  @op @2
-* Mul                  @op @2
-#| Lor            @op @top
-& Land           @op @top
-?\? ALT                @op  
-~ Lnot           @op @1 @top
-^ Xor           @op @2 @top
-++ Next                @op @1 @top
--- Prev                @op @1 @top
+== Eq                  @op @A2   @P9
+!= Neq                 @op @A2   @P9
+< Lt                   @op @A2   @P9
+<= Lte                 @op @A2   @P9
+> Gt                   @op @A2   @P9
+>= Gte                 @op @A2   @P9
 
-NUM                    @system @top @const
-STR                    @system @top @const
-TSTR                   @system @top @const
-FMTSTR                 @system @top
+|  Lor                  @op @top  @P11
+%% Fmt                 @op @2    @P12 @A2
+
+<< Lshift              @op @2    @P15
+>> Rshift              @op @2    @P15
++ Add                  @op @top  @P20
+- Sub                  @op @top  @P20  @A2
+/ Div                  @op @2    @P24  @A2
+% Mod                  @op @2    @P24  @A2
+* Mul                  @op @2    @P24  @A2
+
+& Land                 @op @top      @P27
+~ Lnot                 @op @A1 @top   @P27
+^ Xor                  @op @A2 @top    @P27
+++ Next                @op @A1 @top  @P28
+-- Prev                @op @A1 @top  @P28
+--- Neg                @op @A1 @top  @P28
+
+NUM                    @system @top @const @P99
+STR                    @system @top @const @P99
+TSTR                   @system @top @const @P99
+EVAL                   @system @top @P99
 LABEL                  @system @top
 METAN                  @system @top @eos
-PROPN                  @system @top
-URN                    @system @top @const
-MT                     @system @top 
-CONSTN                 @system @top @const
-TYPEN                  @system @top @const
-CMETHODN               @system @top @const
-NAME                   @system @top @const
-MN                     @system
-FN                     @system
-CID                    @system
-CONST                  @system @top @const @typed
-MTDMPR                 @system @top @typed
-EBPIDX                 @system @typed
-SFPIDX                 @system @typed
-FLDIDX                 @system @typed
-OBJIDX                 @system @typed
-SYSCONST               @system @typed
+PROPN                  @system @top @P99
+URN                    @system @top @const @P99
+MT                     @system @top @P99
+CONSTN                 @system @top @const @P99
+TYPEN                  @system @top @const @P99
+CMETHODN               @system @top @const @P99
+NAME                   @system @top @const @P99
+MN                     @system @P99
+FN                     @system @P99
+CID                    @system @P99
+CONST                  @system @top @const @typed @P99
+MTDMPR                 @system @top @typed @P99
+EBPIDX                 @system @typed @P99
+SFPIDX                 @system @typed @P99
+FLDIDX                 @system @typed @P99
+OBJIDX                 @system @typed @P99
+SYSCONST               @system @typed @P99
 _ ASIS                 @top    @typed
 ERR                    @system 
 EOT                    @system @eos
@@ -201,7 +205,7 @@ mod     TT_MOD          @alias
 #any     TT_TYPEN        @alias
 
 int     Int!            @alias
-long    Int64!          @alias
+long    Int!            @alias
 float   Float!          @alias
 double  Float!          @alias
 boolean Boolean!        @alias
@@ -259,6 +263,14 @@ class Token:
         return op in self.options
     
     def array(self, op):
+        if op == '@P':
+            for op in self.options:
+                if op.startswith('@P'): return op[2:]
+            return '0'
+        if op == '@A':
+            for op in self.options:
+                if op.startswith('@A'): return op[2:]
+            return '0'
         if op == '@name': return '"%s"' % self.token
         if op == '@op': return self.opFunc
         if op == '@eos': 
@@ -433,7 +445,7 @@ static %s knh_%s_array_%s[] = {''' % (ctype, ttype, fn))
     f.write('''
 %s
 {
-    DEBUG_ASSERT(t < KNH_%s_MAXSIZ);
+    KNH_ASSERT(t < KNH_%s_MAXSIZ);
     return knh_%s_array_%s[t];
 }
 ''' % (func, ttype.upper(), ttype, fn))
@@ -694,11 +706,11 @@ new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc)
     case TT_SEMICOLON:
         return new_StmtDONE(ctx);''')
             continue
-        if tk.TT == 'TT_QUESTION' :
-            f.write('''
-    case TT_QUESTION:
-        return new_StmtMAN(ctx, tc);''')
-            continue
+#        if tk.TT == 'TT_QUESTION' :
+#            f.write('''
+#//   case TT_QUESTION:
+#//      return new_StmtMAN(ctx, tc);''')
+#            continue
         if tk.TT == 'TT_ERR' :
             f.write('''
     case TT_ERR:
@@ -784,6 +796,8 @@ def gen_stmt(bdir):
     write_chapter(f,'[token]')
     write_array(f, 'char*', 'token', 'tochar', TT_LIST, '@name')
     write_array(f, 'knh_methodn_t', 'token', 'tomethodn', TT_LIST, '@op')
+    write_array(f, 'int', 'token', 'getOpPriority', TT_LIST, '@P')
+    write_array(f,  'int', 'token', 'getOpSize', TT_LIST, '@A')
     write_array(f, 'knh_bool_t', 'token', 'isEndOfStmt', TT_LIST, '@eos')
     write_tparser_c(f)
     write_chktoken_c(f)
