@@ -228,7 +228,7 @@ METHOD knh__DictSet_opLshift(Ctx *ctx, knh_sfp_t *sfp)
 	if(!knh_Object_isImmutable(o)) {
 		knh_sfp_t *v = sfp + 1;
 		knh_vargc_t ac = knh_sfp_argc(ctx, v);
-		int i;
+		size_t i;
 		for(i = 0; i < ac; i++) {
 			if(IS_NULL(v[1].o)) continue;
 			knh_DictSet_append(ctx, o, v[i].s, 1);
