@@ -35,100 +35,15 @@
 extern "C" {
 #endif
 
-/* ======================================================================== */
-/* [drivers] */
-
-static knh_outptr_t* knh_OutputStream_open__NOP(Ctx *ctx, knh_bytes_t n, char *mode);
-static void knh_OutputStream_putc__NOP(Ctx *ctx, knh_outptr_t *ptr, int ch);
-static size_t knh_OutputStream_write__NOP(Ctx *ctx, knh_outptr_t *ptr, char *buf, size_t bufsiz);
-static void knh_OutputStream_flush__NOP(Ctx *ctx, knh_outptr_t *ptr);
-static void knh_OutputStream_close__NOP(Ctx *ctx, knh_outptr_t *ptr);
-
-static knh_outptr_t* knh_OutputStream_open__FILE(Ctx *ctx, knh_bytes_t file, char *mode);
-static void knh_OutputStream_putc__FILE(Ctx *ctx, knh_outptr_t *ptr, int ch);
-static size_t knh_OutputStream_write__FILE(Ctx *ctx, knh_outptr_t *ptr, char *buf, size_t bufsiz);
-static void knh_OutputStream_flush__FILE(Ctx *ctx, knh_outptr_t *ptr);
-static void knh_OutputStream_close__FILE(Ctx *ctx, knh_outptr_t *ptr);
-
-static void knh_OutputStream_putc__Bytes(Ctx *ctx, knh_outptr_t *ptr, int ch);
-static size_t knh_OutputStream_write__Bytes(Ctx *ctx, knh_outptr_t *ptr, char *buf, size_t bufsiz);
-static void knh_OutputStream_close__Bytes(Ctx *ctx, knh_outptr_t *ptr);
-
-static knh_outptr_drvapi_t OUT__NOP = {
-	KNH_DRVAPI_TYPE__OUTPUTSTREAM, "nop",
-	knh_OutputStream_open__NOP,
-	knh_OutputStream_putc__NOP,
-	knh_OutputStream_write__NOP,
-	knh_OutputStream_flush__NOP,
-	knh_OutputStream_close__NOP
-};
-
-static knh_outptr_drvapi_t OUT__FILE = {
-	KNH_DRVAPI_TYPE__OUTPUTSTREAM, "file",
-	knh_OutputStream_open__FILE,
-	knh_OutputStream_putc__FILE,
-	knh_OutputStream_write__FILE,
-	knh_OutputStream_flush__FILE,
-	knh_OutputStream_close__FILE
-};
-
-static knh_outptr_drvapi_t OUT__stdout = {
-	KNH_DRVAPI_TYPE__OUTPUTSTREAM, "stdout",
-	knh_OutputStream_open__NOP,
-	knh_OutputStream_putc__FILE,
-	knh_OutputStream_write__FILE,
-	knh_OutputStream_flush__FILE,
-	knh_OutputStream_close__NOP
-};
-
-static knh_outptr_drvapi_t OUT__Bytes = {
-	KNH_DRVAPI_TYPE__OUTPUTSTREAM, "Bytes",
-	knh_OutputStream_open__NOP,
-	knh_OutputStream_putc__Bytes,
-	knh_OutputStream_write__Bytes,
-	knh_OutputStream_flush__NOP,
-	knh_OutputStream_close__Bytes
-};
-
-/* ------------------------------------------------------------------------ */
-
-static
-knh_outptr_drvapi_t *knh_System_getOutputStreamDriver(Ctx *ctx, knh_bytes_t name)
-{
-	knh_outptr_drvapi_t *p = (knh_outptr_drvapi_t *)knh_System_getDRVAPI(ctx, KNH_DRVAPI_TYPE__OUTPUTSTREAM, name);
-	if(p == NULL) {
-		KNH_WARNING(ctx, "OutputStream: unsupported scheme '%s'", name);
-		p = &OUT__NOP;
-	}
-	return p;
-}
-
-/* ------------------------------------------------------------------------ */
-
-KNHAPI(void) konoha_addOutputStreamDriver(Ctx *ctx, char *alias, knh_outptr_drvapi_t *d)
-{
-	KNH_TDRVAPI(ctx, alias, (knh_drvapi_t*)d);
-}
-
-/* ------------------------------------------------------------------------ */
-
-void KNHINIT init_OutputStream(Ctx *ctx)
-{
-	konoha_addOutputStreamDriver(ctx, NULL, &OUT__NOP);
-	konoha_addOutputStreamDriver(ctx, NULL, &OUT__FILE);
-	konoha_addOutputStreamDriver(ctx, NULL, &OUT__stdout);
-	konoha_addOutputStreamDriver(ctx, NULL, &OUT__Bytes);
-}
-
 /* ======================================================================== */
 /* [structs] */
 
 void
 knh_OutputStream_struct_init(Ctx *ctx, knh_OutputStream_struct *b, int init, Object *cs)
 {
-	b->outptr = NULL;
-	KNH_INITv(b->ba, KNH_NULL);
-	b->apis = OUT__NOP;
+	b->fd = -1;
+	b->driver = knh_System_getDefaultIODriver();
+	KNH_INITv(b->ba, new_Bytes(ctx, 8));
 	KNH_INITv(b->bconv, KNH_NULL);
 	KNH_INITv(b->enc, TS_ENCODING);
 	KNH_INITv(b->urn, TS_DEVNULL);
@@ -152,9 +67,9 @@ knh_OutputStream_struct_init(Ctx *ctx, knh_OutputStream_struct *b, int init, Obj
 void
 knh_OutputStream_struct_traverse(Ctx *ctx, knh_OutputStream_struct *b, f_traverse gc)
 {
-	if(IS_SWEEP(gc) && b->outptr != NULL) {
-		b->apis.fclose(ctx, b->outptr);
-		b->outptr = NULL;
+	if(IS_SWEEP(gc) && b->fd != -1) {
+		b->driver.fclose(ctx, b->fd);
+		b->fd = -1;
 	}
 	gc(ctx, UP(b->ba));
 	gc(ctx, UP(b->enc));
@@ -171,26 +86,30 @@ Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *mo
 {
 	knh_bytes_t fname = knh_String_tobytes(urn);
 	knh_index_t loc = knh_bytes_index(fname, ':');
+
 	if(loc == -1 || (loc == 1 && isalpha(fname.buf[0]))) {  /* 'C:/' */
-		DP(o)->apis = *(knh_System_getOutputStreamDriver(ctx, STEXT("file")));
+		DP(o)->driver = *(knh_System_getIODriver(ctx, STEXT("file")));
 	}
 	else {
-		DP(o)->apis = *(knh_System_getOutputStreamDriver(ctx, knh_bytes_first(fname, loc)));
+		DP(o)->driver = *(knh_System_getIODriver(ctx, knh_bytes_first(fname, loc)));
 	}
 	if(IS_NULL(mode)) {
-		DP(o)->outptr = DP(o)->apis.fopen(ctx, fname, "w");
+		DP(o)->fd = DP(o)->driver.fopen(ctx, UP(o), fname, "w");
 	}
 	else {
-		DP(o)->outptr = DP(o)->apis.fopen(ctx, fname, knh_String_tochar(mode));
+		DP(o)->fd = DP(o)->driver.fopen(ctx, UP(o), fname, knh_String_tochar(mode));
 	}
-	if(DP(o)->outptr == NULL) {
+
+	if(DP(o)->fd != -1) {
+		KNH_SETv(ctx, DP(o)->ba, new_Bytes(ctx, DP(o)->driver.bufsiz));
+		knh_OutputStream_setBOL(o,1);
+	}
+	else {
 		char buff[FILENAME_BUFSIZ];
 		knh_snprintf(buff, sizeof(buff), "IO!!: cannot open %s", (char*)fname.buf);
-		DP(o)->apis = OUT__NOP;
+		DP(o)->driver = knh_System_getDefaultIODriver();
 		return (Object*)new_Nue__s(ctx, buff);
 	}
-	knh_OutputStream_setBOL(o,1);
-	knh_OutputStream_setAutoFlush(o,1);
 	return (Object*)o;
 }
 
@@ -199,7 +118,13 @@ Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *mo
 
 void knh_OutputStream_putc(Ctx *ctx, OutputStream *o, int ch)
 {
-	DP(o)->apis.fputc(ctx, DP(o)->outptr, ch);
+	Bytes *ba = DP(o)->ba;
+	KNH_ASSERT(IS_Bytes(ba));
+	knh_Bytes_putc(ctx, ba, ch);
+	if(!knh_OutputStream_isStoreBuffer(o) && ba->size > DP(o)->driver.bufsiz) {
+		DP(o)->driver.fwrite(ctx, DP(o)->fd, (char*)(ba)->buf, (ba)->size);
+		knh_Bytes_clear(ba);
+	}
 	DP(o)->size++;
 }
 
@@ -207,7 +132,13 @@ void knh_OutputStream_putc(Ctx *ctx, OutputStream *o, int ch)
 
 void knh_OutputStream_write(Ctx *ctx, OutputStream *o, knh_bytes_t buf)
 {
-	DP(o)->apis.fwrite(ctx, DP(o)->outptr, (char*)buf.buf, buf.len);
+	Bytes *ba = DP(o)->ba;
+	KNH_ASSERT(IS_Bytes(ba));
+	knh_Bytes_write(ctx, ba, buf);
+	if(!knh_OutputStream_isStoreBuffer(o) && ba->size > DP(o)->driver.bufsiz) {
+		DP(o)->driver.fwrite(ctx, DP(o)->fd, (char*)(ba)->buf, (ba)->size);
+		knh_Bytes_clear(ba);
+	}
 	DP(o)->size += buf.len;
 }
 
@@ -216,7 +147,11 @@ void knh_OutputStream_write(Ctx *ctx, OutputStream *o, knh_bytes_t buf)
 
 void knh_OutputStream_flush(Ctx *ctx, OutputStream *o)
 {
-	DP(o)->apis.fflush(ctx, DP(o)->outptr);
+	if(!knh_OutputStream_isStoreBuffer(o)) {
+		Bytes *ba = DP(o)->ba;
+		DP(o)->driver.fwrite(ctx, DP(o)->fd, (char*)(ba)->buf, (ba)->size);
+		knh_Bytes_clear(ba);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -224,9 +159,17 @@ void knh_OutputStream_flush(Ctx *ctx, OutputStream *o)
 
 void knh_OutputStream_close(Ctx *ctx, OutputStream *o)
 {
-	DP(o)->apis.fclose(ctx, DP(o)->outptr);
-	DP(o)->outptr = NULL;
-	DP(o)->apis = OUT__NOP;
+	f_io_close f = DP(o)->driver.fclose;
+	DP(o)->driver = knh_System_getDefaultIODriver();
+	f(ctx, DP(o)->fd);
+	DP(o)->fd = -1;
+}
+
+/* ------------------------------------------------------------------------ */
+
+int knh_OutputStream_isClosed(OutputStream *o)
+{
+	return (DP(o)->fd == -1);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -262,14 +205,12 @@ void knh_OutputStream_print_(Ctx *ctx, OutputStream *o, knh_bytes_t str, knh_boo
 			knh_write_BOL(ctx, o);
 		}
 		if(IS_NULL(DP(o)->bconv)) {
-			DP(o)->apis.fwrite(ctx, DP(o)->outptr, (char*)str.buf, str.len);
-			DP(o)->size += str.len;
+			knh_OutputStream_write(ctx, o, str);
 		}
 		else {
 			Bytes *ba = knh_Context_openBConvBuf(ctx);
-			DP(o)->size += knh_BytesConv_conv(ctx, DP(o)->bconv, str, ba);
-			//DBG2_(DP(o)->apis.fwrite(ctx, DP(o)->outptr, (char*)str.buf, str.len);)
-			DP(o)->apis.fwrite(ctx, DP(o)->outptr, knh_Bytes_tochar(ba), ba->size);
+			knh_BytesConv_conv(ctx, DP(o)->bconv, str, ba);
+			knh_OutputStream_write(ctx, o, knh_Bytes_tobytes(ba));
 			knh_Context_closeBConvBuf(ctx, ba);
 		}
 	}
@@ -280,162 +221,59 @@ void knh_OutputStream_print_(Ctx *ctx, OutputStream *o, knh_bytes_t str, knh_boo
 
 
 /* ======================================================================== */
-/* ======================================================================== */
-/* [nop] */
 
-static
-knh_outptr_t* knh_OutputStream_open__NOP(Ctx *ctx, knh_bytes_t n, char *mode)
-{
-	return NULL;
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_OutputStream_putc__NOP(Ctx *ctx, knh_outptr_t *ptr, int ch)
-{
-
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-size_t knh_OutputStream_write__NOP(Ctx *ctx, knh_outptr_t *ptr, char *buf, size_t bufsiz)
-{
-	return bufsiz;
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_OutputStream_flush__NOP(Ctx *ctx, knh_outptr_t *ptr)
-{
-
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_OutputStream_close__NOP(Ctx *ctx, knh_outptr_t *ptr)
-{
-
-}
-
-/* ======================================================================== */
-/* [FILE] */
-
-static
-knh_outptr_t* knh_OutputStream_open__FILE(Ctx *ctx, knh_bytes_t file, char *mode)
-{
-	char buf[FILENAME_BUFSIZ];
-	knh_format_ospath(buf, sizeof(buf), file);
-	{
-		FILE *fp = knh_fopen(buf, mode);
-		if(fp == NULL) {
-			KNH_THROWf(ctx, "IO!!: cannot open: %s, mode='%s'\n", (char*)file.buf, mode);
-			return NULL;
-		}
-		return (knh_outptr_t*)fp;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_OutputStream_putc__FILE(Ctx *ctx, knh_outptr_t *ptr, int ch)
-{
-	fputc(ch, (FILE*)ptr);
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-size_t knh_OutputStream_write__FILE(Ctx *ctx, knh_outptr_t *ptr, char *buf, size_t bufsiz)
-{
-	return fwrite(buf, 1, bufsiz, (FILE*)ptr);
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_OutputStream_flush__FILE(Ctx *ctx, knh_outptr_t *ptr)
-{
-	fflush((FILE*)ptr);
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_OutputStream_close__FILE(Ctx *ctx, knh_outptr_t *ptr)
-{
-	knh_fclose((FILE*)ptr);
-}
-
-/* ------------------------------------------------------------------------ */
-
-OutputStream *new_OutputStream__FILE(Ctx *ctx, FILE *fp)
+OutputStream *new_OutputStream__stdio(Ctx *ctx, FILE *fp)
 {
 	OutputStream* o = (OutputStream*)new_Object_malloc(ctx, FLAG_OutputStream, CLASS_OutputStream, sizeof(knh_OutputStream_struct));
 	knh_OutputStream_struct_init(ctx, DP(o), 0, NULL);
 	knh_OutputStream_setBOL(o, 1);
-	knh_OutputStream_setAutoFlush(o, 1);
-	DP(o)->outptr = (knh_outptr_t*)fp;
+
+	KNH_ASSERT(fp == stdout || fp == stderr);
+
+	DP(o)->driver = knh_System_getStdIODriver();
+	DP(o)->fd = (knh_io_t)fp;
+
 	if(fp == stdout) {
 		KNH_SETv(ctx, DP(o)->urn, TS_DEVSTDOUT);
-		DP(o)->apis = OUT__stdout;
+		knh_OutputStream_setAutoFlush(o, 1);
 	}
 	else if(fp == stderr) {
 		KNH_SETv(ctx, DP(o)->urn, TS_DEVSTDERR);
-		DP(o)->apis = OUT__stdout;
-	}
-	else {
-		DP(o)->apis = OUT__FILE;
+		knh_OutputStream_setAutoFlush(o, 1);
 	}
 	return o;
 }
 
-/* ======================================================================== */
-/* [Bytes] */
-
-static
-void knh_OutputStream_putc__Bytes(Ctx *ctx, knh_outptr_t *ptr, int ch)
-{
-	OutputStream *o = (OutputStream*)ptr;
-	knh_Bytes_putc(ctx, DP(o)->ba, ch);
-}
-
 /* ------------------------------------------------------------------------ */
 
-static
-size_t knh_OutputStream_write__Bytes(Ctx *ctx, knh_outptr_t *ptr, char *buf, size_t bufsiz)
-{
-	OutputStream *o = (OutputStream*)ptr;
-	knh_Bytes_write(ctx, DP(o)->ba, B2(buf, bufsiz));
-	return bufsiz;
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_OutputStream_close__Bytes(Ctx *ctx, knh_outptr_t *ptr)
-{
-	OutputStream *o = (OutputStream*)ptr;
-	KNH_SETv(ctx, DP(o)->ba, KNH_NULL);
-}
-
-/* ------------------------------------------------------------------------ */
-
-OutputStream *new_OutputStream__Bytes(Ctx *ctx, Bytes *ba)
+KNHAPI(OutputStream*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode)
 {
 	OutputStream* o = (OutputStream*)new_Object_malloc(ctx, FLAG_OutputStream, CLASS_OutputStream, sizeof(knh_OutputStream_struct));
 	knh_OutputStream_struct_init(ctx, DP(o), 0, NULL);
 	knh_OutputStream_setBOL(o, 1);
-	//knh_OutputStream_setAutoFlush(o, 1);
-	if(!IS_NULL(ba)) {
-		KNH_SETv(ctx, DP(o)->ba, ba);
-		DP(o)->apis = OUT__Bytes;
+
+	DP(o)->driver = *(knh_System_getIODriver(ctx, STEXT("file")));
+	DP(o)->fd = DP(o)->driver.fopen(ctx, UP(o), file, mode);
+	if(DP(o)->fd != -1) {
+		KNH_SETv(ctx, DP(o)->ba, new_Bytes(ctx, DP(o)->driver.bufsiz));
+		KNH_SETv(ctx, DP(o)->urn, new_String(ctx, file, NULL));
 	}
-	DP(o)->outptr = (knh_outptr_t*)o;
+	else {
+		DP(o)->driver = knh_System_getDefaultIODriver();
+	}
+	return o;
+}
+
+/* ------------------------------------------------------------------------ */
+
+KNHAPI(OutputStream*) new_BytesOutputStream(Ctx *ctx, Bytes *ba)
+{
+	OutputStream* o = (OutputStream*)new_Object_malloc(ctx, FLAG_OutputStream, CLASS_OutputStream, sizeof(knh_OutputStream_struct));
+	knh_OutputStream_struct_init(ctx, DP(o), 0, NULL);
+	knh_OutputStream_setBOL(o, 1);
+	KNH_ASSERT(IS_Bytes(ba));
+	KNH_SETv(ctx, DP(o)->ba, ba);
+	knh_OutputStream_setStoreBuffer(o, 1);
 	return o;
 }
 
