@@ -57,20 +57,20 @@ METHOD knh__OutputStream_write(Ctx *ctx, knh_sfp_t *sfp)
 {
 	OutputStream *o = (OutputStream*)sfp[0].o;
 	knh_bytes_t buf = knh_Bytes_tobytes((Bytes*)sfp[1].o);
-	size_t s = IS_NULL(sfp[2].o) ? 0 : knh_array_index(ctx, ARG_int(sfp[2]), buf.len);
-	if(IS_NULL(sfp[3].o)) {
-		size_t len = buf.len - s;
-		DP(o)->apis.fwrite(ctx, DP(o)->outptr, (char*)(buf.buf + s), len);
-		DP(o)->size += len;
-	}
-	else {
-		size_t len = (sfp[3].i)->value;
-		if(s + len > buf.len) {
-			len = buf.len - s;
-		}
-		DP(o)->apis.fwrite(ctx, DP(o)->outptr, (char*)(buf.buf + s), len);
-		DP(o)->size += len;
-	}
+	size_t offset = IS_NULL(sfp[2].o) ? 0 : knh_array_index(ctx, ARG_int(sfp[2]), buf.len);
+	size_t len = IS_NULL(sfp[3].o) ? (buf.len - offset) : ARG_int(sfp[3]);
+	if(offset + len > buf.len) len = buf.len - offset;
+	buf.buf = &(buf.buf[offset]);
+	buf.len = len;
+	knh_OutputStream_write(ctx, o, buf);
+}
+
+/* ------------------------------------------------------------------------ */
+/* @method Boolean! OutputStream.isClosed() */
+
+METHOD knh__OutputStream_isClosed(Ctx *ctx, knh_sfp_t *sfp)
+{
+	KNH_RETURN_Boolean(ctx, sfp, knh_OutputStream_isClosed((OutputStream*)sfp[0].o));
 }
 
 /* ------------------------------------------------------------------------ */
