@@ -75,10 +75,7 @@ knh_index_t knh_bytes_indexOf(knh_bytes_t base, knh_bytes_t delim)
 	size_t i, j;
 	//TODO: handle, when delim is NULL. ""
 	if(delim.len > base.len) {
-		char buf[80];
-		//TODO: Exception!!!
-		knh_snprintf(buf, sizeof(buf), "Arg String is Invalid size: %d (> %d)", (int)delim.len, (int)base.len);
-		DBG2_P("%s", buf);
+		// Error. array index is out of boundary
 		return -1;
 	}
 	for (i = 0; i <= base.len - delim.len; i++) {
@@ -95,8 +92,15 @@ knh_index_t knh_bytes_indexOf(knh_bytes_t base, knh_bytes_t delim)
 /* @method[CONST] Int! String.indexOf(String! s) */
 METHOD knh__String_indexOf(Ctx *ctx, knh_sfp_t *sfp)
 {
-
-	KNH_RETURN(ctx, sfp, new_Int(ctx, knh_bytes_indexOf(knh_String_tobytes(sfp[0].s), knh_String_tobytes(sfp[1].s))));
+	knh_bytes_t base = knh_String_tobytes(sfp[0].s);
+	knh_bytes_t delim = knh_String_tobytes(sfp[1].s);
+	int loc = knh_bytes_indexOf(base, delim);
+	if (knh_String_isASCII(sfp[0].s) || loc == -1) {
+		KNH_RETURN_Int(ctx, sfp, loc);
+	} else {
+		base.len = (size_t)loc;
+		KNH_RETURN_Int(ctx, sfp, knh_bytes_mlen(base));
+	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -410,21 +414,22 @@ METHOD knh__String_opOffset(Ctx *ctx, knh_sfp_t *sfp)
 			KNH_RETURN(ctx, sfp, new_String(ctx, knh_bytes_last(base, offset), sfp[0].s));
 		}
 		else {
-			knh_bytes_t sub = knh_bytes_offlen(base, offset, knh_array_index(ctx, ARG_int(sfp[2]), base.len - offset ));
+			knh_bytes_t sub = knh_bytes_offlen(base, offset, knh_array_index(ctx, ARG_int(sfp[2]), base.len - offset + 1 ));
 			KNH_RETURN(ctx, sfp, new_String(ctx, sub, sfp[0].s));
 		}
 	}
 	else {
-		TODO_THROW(ctx);
-//		size_t mlen = knh_bytes_mlen(base);
-//		size_t offset = IS_NULL(sfp[1].o) ? 0 : knh_array_index(ctx, ARG_int(sfp[1]), mlen);
-//		if(IS_NULL(sfp[2].o)) {
-//			METHOD_RETURN(ctx, sfp, new_String(ctx, knh_bytes_last(base, offset), sfp[0].s));
-//		}
-//		else {
-//			knh_bytes_t sub = knh_bytes_mofflen(base, offset, ARG_int(sfp[2]));
-//			METHOD_RETURN(ctx, sfp, new_String(ctx, sub, sfp[0].s));
-//		}
+		// multibytes
+		DBG2_P("hello");
+		size_t mlen = knh_bytes_mlen(base);
+		size_t offset = IS_NULL(sfp[1].o) ? 0 : knh_array_index(ctx, ARG_int(sfp[1]), mlen);
+		if(IS_NULL(sfp[2].o)) {
+			KNH_RETURN(ctx, sfp, new_String(ctx, knh_bytes_last(base, offset), sfp[0].s));
+		}
+		else {
+			knh_bytes_t sub = knh_bytes_mofflen(base, offset, ARG_int(sfp[2]));
+			KNH_RETURN(ctx, sfp, new_String(ctx, sub, sfp[0].s));
+		}
 	}
 }
 
