@@ -790,7 +790,7 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh
 	//DEBUG3("reqc=%s", CLASSN(reqc));
 	if(knh_Token_isExceptionType(DP(stmt)->tokens[1])) {
 		mtd_cid = CLASS_Exception;
-		TODO();
+		mn = METHODN_new__init;
 	}
 	else {
 		mtd_cid = knh_Token_toclass(ctx, DP(stmt)->tokens[1], CLASS_unknown, ns);
@@ -836,8 +836,9 @@ Term *knh_StmtNEW_typing(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh
 	}
 
 	DEBUG_ASSERT_cid(mtd_cid);
-	KNH_SETv(ctx, DP(stmt)->terms[1], new_TokenCONST(ctx, FL(DP(stmt)->terms[0]), UP(knh_tClass[mtd_cid].class_cid)));
-	//DEBUG3("Typing for %s", CLASSN(mtd_cid));
+	if(!knh_Token_isExceptionType(DP(stmt)->tokens[1])) {
+		KNH_SETv(ctx, DP(stmt)->terms[1], new_TokenCONST(ctx, FL(DP(stmt)->terms[0]), UP(knh_tClass[mtd_cid].class_cid)));
+	}
 	knh_StmtPARAMS_typing(ctx, stmt, cpr, ns, CLASS_TONNTYPE(mtd_cid), mtd);
 	return TM(stmt);
 }
@@ -1533,22 +1534,41 @@ void knh_StmtOP_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_typ
 void knh_StmtNEW_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
 {
 	Token *mtdtk = DP(stmt)->tokens[0];
-	Object *value = knh_Term_constValueNULL(ctx, DP(stmt)->terms[1], CLASS_Any);
 	DEBUG3_ASSERT(SP(mtdtk)->tt == TT_MTDMPR);
-	DEBUG3_ASSERT(value != NULL && IS_Class(value));
 	Method *mtd = DP(mtdtk)->mtd;
-	knh_class_t mtd_cid = ((Class*)value)->cid;
+	knh_class_t mtd_cid;
+	if(knh_Token_isExceptionType(DP(stmt)->tokens[1])) {
+		mtd_cid = CLASS_Exception;
+	}
+	else {
+		Object *value = knh_Term_constValueNULL(ctx, DP(stmt)->terms[1], CLASS_Any);
+		DEBUG3_ASSERT(value != NULL && IS_Class(value));
+		mtd_cid = ((Class*)value)->cid;
+	}
+
 	int i;
 	knh_flag_t flag = 0;
 
 	KNH_ASM_PPUSH(ctx, cpr, UP(mtdtk), level);
 	KNH_ASM_PUSH__OBJ(ctx, cpr, (Object*)KNH_NULL);
+	if(knh_Token_isExceptionType(DP(stmt)->tokens[1])) {
+		Object *s = DP(DP(stmt)->tokens[1])->data;
+		KNH_ASSERT(IS_String(s));
+		KNH_ASM_PUSH__OBJ(ctx, cpr, s);
+	}
+
 	for(i = 2; i < DP(stmt)->size; i++) {
 		knh_type_t reqt = knh_pmztype_totype(ctx, knh_Method_ptype(mtd, i - 2), mtd_cid);
 		knh_Stmt_terms_cmplpush(ctx, stmt, i, cpr, ns, reqt, level + i);
 		if(SP(stmt)->stt == STT_ERR) { return ;}
 	}
-	KNH_ASM_NEW(ctx, cpr, DP(stmt)->size - 1, flag, mtd_cid);
+
+	if(knh_Token_isExceptionType(DP(stmt)->tokens[1])) {
+		KNH_ASM_NEW(ctx, cpr, DP(stmt)->size, flag, mtd_cid);
+	}
+	else {
+		KNH_ASM_NEW(ctx, cpr, DP(stmt)->size - 1, flag, mtd_cid);
+	}
 }
 
 /* ------------------------------------------------------------------------ */
