@@ -273,6 +273,27 @@ void knh_StmtIF_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_typ
 	}
 }
 
+/* ======================================================================== */
+/* [IF] */
+
+#ifndef StmtMACRO
+#define StmtIF_bool(stmt)               DP(stmt)->terms[0]
+#define StmtIF_truecase(stmt)           DP(stmt)->stmts[1]
+#define StmtIF_falsecase(stmt)          DP(stmt)->stmts[2]
+#endif
+
+void knh_StmtSWITCH_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns,int level)
+{
+	knh_Compiler_perror(ctx, cpr, KMSG_EFUTURE, "switch");
+	knh_Stmt_done(ctx, stmt);
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_StmtSWITCH_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
+{
+}
+
 /* ------------------------------------------------------------------------ */
 
 void knh_StmtALT_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
@@ -716,8 +737,93 @@ void knh_StmtFOR_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_ty
 	KNH_ASM_LABEL__b(ctx, cpr, B(lbend));
 }
 
+/* ======================================================================== */
+/* [TRY] */
+
+void knh_StmtTRY_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int level)
+{
+//	if(level > 1) {
+//		knh_Compiler_declareVariable(ctx, cpr, 0, TYPE_Any, knh_tName_getFieldnOnStack(ctx, level), KNH_NULL);
+//		knh_Stmt_name(ctx, StmtTRY_try(stmt), cpr, ns, level+1);
+//		{
+//			Stmt *instmt = DP(stmt)->stmts[TRY_catch];
+//			Token *tk = NULL;
+//			while(SP(instmt)->stt == STT_CATCH) {
+//				if(tk == NULL) {
+//					tk = DP(instmt)->tokens[1];
+//					knh_Compiler_declareVariable(ctx, cpr, 0, NNTYPE_Exception, knh_Token_tofieldn(tk), KNH_NULL);
+//				}
+//				else {
+//					if(knh_bytes_equals(knh_Token_tobytes(tk), knh_Token_tobytes(DP(instmt)->tokens[1]))) {
+//						knh_Compiler_perror(ctx, cpr, KMSG_DIFFENAME, NULL);
+//						continue;
+//					}
+//				}
+//				KNH_ASSERT(IS_Stmt(DP(instmt)->stmts[2]));
+//				knh_Stmt_name(ctx, DP(instmt)->stmts[2], cpr, ns, level+1);
+//				instmt = DP(instmt)->next;
+//			}
+//		}
+//		knh_Stmt_finally(ctx, StmtTRY_finally(stmt), cpr, ns, level+1);
+//	}
+}
+
 /* ------------------------------------------------------------------------ */
 
+void knh_StmtTRY_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
+{
+//	char lbcatch[LABEL_BUFSIZ];
+//	knh_snprintf(lbcatch, sizeof(lbcatch), "_CATCH_%d", knh_Compiler_llstep(cpr));
+//	char lbend[LABEL_BUFSIZ];
+//	knh_snprintf(lbend, sizeof(lbend), "_END_%d", knh_Compiler_llstep(cpr));
+//
+//	knh_index_t hdr_sfp = knh_Compiler_indexOfVariable(cpr, knh_tName_getFieldnOnStack(ctx, level));
+//	knh_index_t e_sfp;
+//	knh_Compiler_setFinallyStmt(ctx, cpr, hdr_sfp, StmtTRY_finally(stmt));
+//
+//	KNH_ASM_TRY_BEGIN(ctx, cpr, B(lbcatch), hdr_sfp, e_sfp);
+//	knh_Stmt_cmpls(ctx, StmtTRY_main(stmt), cpr, ns, level+1);
+//	KNH_ASM_TRY_END_FINALLY(ctx, cpr);
+//	KNH_ASM_JUMP__b(ctx, cpr, B(lbend));
+//
+//	KNH_ASM_LABEL__b(ctx, cpr, B(lbcatch));
+//	{
+//		Stmt *instmt = DP(stmt)->stmts[TRY_catch];
+//		while(SP(instmt)->stt == STT_CATCH) {
+//			knh_snprintf(lbcatch, sizeof(lbcatch), "_CATCH_%d", knh_Compiler_llstep(cpr));
+//			KNH_ASM_CATCH(ctx, cpr, B(lbcatch), e_sfp, knh_Token_tobytes(DP(stmt)->tokens[0]));
+//			KNH_ASM_TRY_END(ctx, cpr);
+//			knh_Stmt_cmpls(ctx, StmtTRY_main(stmt), cpr, ns, level+1);
+//			KNH_ASM_LABEL__b(ctx, cpr, B(lbcatch));
+//			instmt = DP(instmt)->next;
+//		}
+//	}
+//	KNH_ASM_TRY_END_FINALLY(ctx, cpr);
+//	KNH_ASM_THROW__sfp(ctx, cpr, e_sfp);
+//
+//	KNH_ASM_LABEL__b(ctx, cpr, B(lbend));
+}
+
+/* ======================================================================== */
+/* [THROW] */
+
+void knh_StmtTHROW_name(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, int level)
+{
+
+}
+
+/* ------------------------------------------------------------------------ */
+
+void knh_StmtTHROW_cmpl(Ctx *ctx, Stmt *stmt, Compiler *cpr, NameSpace *ns, knh_type_t reqt, int level)
+{
+	KNH_ASSERT(DP(stmt)->size > 0);
+	knh_Stmt_terms_cmpl(ctx, stmt, 0, cpr, ns, NNTYPE_Exception, 0 /* MC_FIRST*/);
+	KNH_ASM_THROW(ctx, cpr, DP(stmt)->terms[0]);
+}
+
+/* ------------------------------------------------------------------------ */
+
+
 #ifdef __cplusplus
 }
 #endif
