@@ -73,7 +73,7 @@ static void KNH_ASM_PUSH_INITLOCALs(Ctx *ctx, Compiler *cpr)
 	Method *mtd = DP(cpr)->method;
 	KNH_ASSERT(IS_Method(mtd));
 	{
-		int i = 1;
+		size_t i = 1;
 		for(;i < knh_Method_psize(mtd) + 1; i++) {
 			if(IS_NOTNULL(DP(cpr)->vars[i].value)) {
 				KNH_ASM_MOVSFP_IFNUL(ctx, cpr, i, DP(cpr)->vars[i].value);
