@@ -918,7 +918,7 @@ Term *new_TermVALUE(Ctx *ctx, Token *tk, int lr)
 			return TM(new_StmtDICTMAP(ctx, (Token*)KNH_NULL, tk));
 		case TT_PROPN:
 			return TM(new_StmtPROPN(ctx, tk, lr));
-		case TT_FMTSTR:
+		case TT_EVAL:
 			TODO();
 		case TT_STR:
 		case TT_NUM:
