@@ -187,14 +187,6 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 
 		case ';':
 			knh_Token_add_space(ctx, tkctx, tbuf, in, join);
-//			if(SP(tkctx->tk)->tt == TT_PARENTHESIS) {
-//				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EFORGET_PARENTHESIS, ")");
-//				tkctx = knh_pstack_backstmt(ctx, tkctx_stack, tkctx);
-//			}
-//			else if(SP(tkctx->tk)->tt == TT_BRANCET) {
-//				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_EFORGET_PARENTHESIS, "]");
-//				tkctx = knh_pstack_backstmt(ctx, tkctx_stack, tkctx);
-//			}
 			knh_Token_padd(ctx, tkctx, new_Token(ctx, 0, DP(in)->fileid, DP(in)->line, knh_char_totoken(ch)));
 			break;
 
@@ -475,10 +467,10 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 	RAWSTR_PART:;
 	KNH_ASSERT(prev == '"' || prev == '\'' || prev == '`');
 	{
-		knh_Bytes_putc(ctx, tbuf.ba, prev);
+		//knh_Bytes_putc(ctx, tbuf.ba, prev);
 		while((ch = knh_InputStream_getc(ctx, in)) != EOF) {
 			if(ch == prev) {
-				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 				goto MAIN_PART;
 			}
 			knh_Bytes_putc(ctx, tbuf.ba, ch);
@@ -487,7 +479,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 			char buf4[4];
 			knh_snprintf(buf4, sizeof(buf4), "%c", prev);
 			knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WUNCLOSED, buf4);
-			knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
+			knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 			goto MAIN_PART_INLOOP;
 		}
 	}/*RAWSTR_PART*/
@@ -496,14 +488,14 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 	QUOTED_PART:;
 	KNH_ASSERT(prev == '"' || prev == '\'' || prev == '`');
 	{
-		knh_Bytes_putc(ctx, tbuf.ba, prev);
+		//knh_Bytes_putc(ctx, tbuf.ba, prev);
 		ch = knh_InputStream_getc(ctx, in);
 		if(ch == prev) {
 			ch = knh_InputStream_getc(ctx, in);
 			if(prev == ch) { /* triple quote */
 				goto TRIPLE_QUOTED_PART;
 			}else {
-				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 				goto MAIN_PART_INLOOP;
 			}
 		}
@@ -536,11 +528,11 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 				char buf4[4];
 				knh_snprintf(buf4, sizeof(buf4), "%c", prev);
 				knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WUNCLOSED, buf4);
-				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 				goto MAIN_PART_INLOOP;
 			}
 			if(ch == prev) {
-				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 				goto MAIN_PART;
 			}
 			knh_Bytes_putc(ctx, tbuf.ba, ch);
@@ -572,8 +564,8 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 				}
 				continue;
 			}
-			if(ch == '\n') {
-				if(knh_Bytes_size(tbuf.ba) - tbuf.pos == 1) {
+			if(ch == '\n' || ch == '\r') {
+				if(knh_Bytes_size(tbuf.ba) - tbuf.pos == 0) {
 					DBG2_P("remove first linefeed at triplequote");
 					continue;
 				}
@@ -583,7 +575,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 				knh_Bytes_unputc(tbuf.ba);
 				knh_Bytes_unputc(tbuf.ba);
 				knh_Bytes_unputc(tbuf.ba);
-				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
+				knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 				goto MAIN_PART;
 			}
 		}
@@ -591,10 +583,10 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 			char buf4[4];
 			knh_snprintf(buf4, sizeof(buf4), "%c%c%c", prev, prev, prev);
 			knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WUNCLOSED, buf4);
-			knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, TT_STR, tbuf, in));
+			knh_Token_padd(ctx, tkctx, new_Token__buffer(ctx, knh_char_totoken(prev), tbuf, in));
 		}
 	}
-	return;
+	return /* EOF */;
 
 	/** COMMENT **/
 	LINE_COMMENT:
@@ -605,7 +597,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 			}
 		}
 	}
-	return ;
+	return /* EOF */;
 
 	BLOCK_COMMENT:
 	{
@@ -622,7 +614,7 @@ void knh_Token_parse(Ctx *ctx, Token *tk, InputStream *in, int join)
 		}
 		knh_perror(ctx, DP(in)->fileid, DP(in)->line, KMSG_WUNCLOSED, "/* ... */");
 	}
-	return ;
+	return /* EOF */;
 }
 
 /* ------------------------------------------------------------------------ */
@@ -779,7 +771,7 @@ Token *new_Token__name(Ctx *ctx, knh_flag_t flag, knh_fileid_t fileid, knh_line_
 				continue;
 			}
 			if(!isalnum(t.buf[i]) && t.buf[i] != ':') {
-				if(t.buf[i] == '.' || tt != TT_CMETHODN) {
+				if(t.buf[i] == '.' && tt == TT_CMETHODN) {
 					*p = '.'; p++;
 					continue;
 				}
@@ -874,27 +866,24 @@ Token *new_Token__buffer(Ctx *ctx, knh_token_t tt, knh_wbuf_t tbuf, InputStream
 {
 	Token *o = new_Token(ctx, 0, DP(in)->fileid, DP(in)->line, tt);
 	knh_bytes_t t = knh_wbuf_tobytes(tbuf);
-	//DEBUG3("tt=%s, '%s'", knh_token_tochar(tt), t.buf);
+	//DBG2_P("tt=%s, '%s', len=%d", knh_token_tochar(tt), t.buf, t.len);
 
-	if(tt == TT_STR) {
-		if(t.buf[0] == '`' && t.len > 1) {
-			TODO3("add `quote`");
-		}
-		else if(t.buf[0] == '\'') {
-			SP(o)->tt = TT_TSTR;
-		}
-		t = knh_bytes_last(t, 1);
-	}
 	if(tt == TT_METAN && islower(t.buf[0])) {
 		t.buf[0] = toupper(t.buf[0]);
 	}
-	if(IS_NULL(DP(in)->bconv)) {
-		KNH_SETv(ctx, DP(o)->data, new_String(ctx, t, NULL));
+
+	if(t.len == 0) {
+		KNH_SETv(ctx, DP(o)->data, TS_EMPTY);
 	}
 	else {
-		KNH_SETv(ctx, DP(o)->data, new_String__bconv(ctx, t, DP(in)->bconv));
+		if(IS_NULL(DP(in)->bconv)) {
+			KNH_SETv(ctx, DP(o)->data, new_String(ctx, t, NULL));
+		}
+		else {
+			KNH_SETv(ctx, DP(o)->data, new_String__bconv(ctx, t, DP(in)->bconv));
+		}
+		knh_wbuf_clear(tbuf);
 	}
-	knh_wbuf_clear(tbuf);
 	return o;
 }
 
@@ -1053,6 +1042,9 @@ knh_token_t knh_char_totoken(int ch)
 		case '%': return TT_MT;
 		case ';': return TT_SEMICOLON;
 		case ',': return TT_COMMA;
+		case '"': return TT_STR;
+		case '\'': return TT_TSTR;
+		case '`': return TT_EVAL;
 	}
 	KNH_ASSERT(ch == -1);
 	return TT_ERR;
