@@ -59,15 +59,6 @@ extern "C" {
 #define KNH_OBJECT_FIELDSIZE 64
 #endif
 
-typedef knh_Object_t* (*knh_fparser)(Ctx *, struct knh_String_t *p);
-
-typedef struct {
-	int   type;
-	char *name;
-	knh_type_t  rtype;
-	knh_fparser parser;
-} knh_parser_drvapi_t;
-
 /* ------------------------------------------------------------------------ */
 /* @class Any Object knh_Any_t @Cyclic */
 /* @class This Any knh_Any_t @Cyclic  @Private */
@@ -247,32 +238,6 @@ typedef struct knh_DictSet_t {
 	size_t size;
 } knh_DictSet_t;
 
-///* ------------------------------------------------------------------------ */
-///* @ class DictMap Object knh_DictMap_struct */
-//
-//
-//typedef struct knh_DictMap {
-//	size_t size;
-//	size_t capacity;
-//	knh_dictmape_t* list;
-//	size_t sorted;
-//} knh_DictMap_struct;
-//
-///* ------------------------------------------------------------------------ */
-///* @ class DictSet Object knh_DictSet_struct */
-//
-//typedef struct knh_dictsete_t {
-//	struct knh_String_t *key;
-//	knh_uint_t  value;
-//} knh_dictsete_t;
-//
-//typedef struct knh_DictSet {
-//	size_t size;
-//	size_t capacity;
-//	knh_dictsete_t* list;
-//	size_t sorted;
-//} knh_DictSet_struct;
-
 /* ------------------------------------------------------------------------ */
 /* @class HashMap Object knh_HashMap_struct @Param2 */
 
@@ -566,32 +531,6 @@ typedef struct knh_FloatUnit {
 /* ------------------------------------------------------------------------ */
 /* @class Regex Object knh_Regex_t */
 
-typedef void knh_regex_t;
-
-#ifndef KNH_REGEX_NMATCH_MAX
-#define KNH_REGEX_NMATCH_MAX    16
-#endif
-
-typedef struct {
-	int rm_so;   /* start of match */
-	int rm_eo;   /* endo of match */
-	char *rm_name;  /* NULL, if not NAMED */
-} knh_regmatch_t;
-
-typedef knh_regex_t* (*knh_fregmalloc)(Ctx *);
-typedef int (*knh_fregcomp)(Ctx *, knh_regex_t *, char *pattern, int flags);
-typedef int (*knh_fregexec)(Ctx *, knh_regex_t *, char *str, size_t nmatch, knh_regmatch_t p[], int flags);
-typedef void (*knh_fregfree)(knh_regex_t *);
-
-typedef struct {
-	int  type;
-	char *name;
-	knh_fregmalloc regmalloc;
-	knh_fregcomp   regcomp;
-	knh_fregexec   regexec;
-	knh_fregfree   regfree;
-} knh_regex_drvapi_t;
-
 typedef struct {
 	knh_hObject_t h;
 	knh_regex_t *reg;
@@ -602,18 +541,6 @@ typedef struct {
 /* ------------------------------------------------------------------------ */
 /* @class BytesConv Object knh_BytesConv_struct @Private */
 
-struct knh_BytesConv_t;
-typedef size_t (*f_bconv)(Ctx *ctx, struct knh_BytesConv_t *o, knh_bytes_t t, knh_Bytes_t *ba);
-
-typedef struct {
-	int  type;
-	char *name;
-	f_bconv      fbconv;
-	f_bconv      fbconv_inverse;
-} knh_bconv_driapi_t;
-
-/* ------------------------------------------------------------------------ */
-
 #ifdef KNH_USING_ICONV
 	#include<iconv.h>
 #endif
@@ -651,30 +578,17 @@ typedef struct knh_StringUnit {
 /* ------------------------------------------------------------------------ */
 /* @class InputStream Object knh_InputStream_struct */
 
-struct knh_InputStream;
-typedef void knh_inptr_t;
-
-typedef knh_inptr_t* (*f_inptr_open)(Ctx *ctx, knh_bytes_t);
-typedef int (*f_inptr_getc)(Ctx *ctx, knh_inptr_t *);
-typedef size_t (*f_inptr_read)(Ctx *ctx, knh_inptr_t *, char *buf, size_t bufsiz);
-typedef void (*f_inptr_close)(Ctx *ctx, knh_inptr_t *);
-
-typedef struct {
-	int type; char *name;
-	f_inptr_open  fopen;
-	f_inptr_getc  fgetc;
-	f_inptr_read  fread;
-	f_inptr_close fclose;
-} knh_inptr_drvapi_t;
-
-typedef knh_inptr_drvapi_t* (*f_inptr_drivers)(Ctx *ctx, knh_bytes_t);
-
 typedef struct knh_InputStream {
-	knh_inptr_t* inptr;
-	knh_Bytes_t* ba;
-	size_t  bapos;
-	size_t  baend;
-	knh_inptr_drvapi_t apis;
+	knh_io_t fd;
+	knh_iodrv_t driver;
+	union {
+		struct knh_Bytes_t *ba;
+		struct knh_String_t *str;
+	};
+	char *buf;
+	size_t bufpos;
+	size_t bufend;
+	size_t bufsiz;
 	knh_String_t *enc;
 	struct knh_BytesConv_t* bconv;
 	knh_String_t*  urn;
@@ -688,29 +602,12 @@ typedef struct knh_InputStream {
 /* @class OutputStream Object knh_OutputStream_struct */
 /* @flag OutputStream.BOL OUT:1 (%s)->h.flag 'is:set:*:*' */
 /* @flag OutputStream.AutoFlush OUT:2 (%s)->h.flag 'is:set:is:set' */
-
-struct knh_OutputStream;
-typedef void knh_outptr_t;
-
-typedef knh_outptr_t* (*f_outptr_open)(Ctx *ctx, knh_bytes_t, char *mode);
-typedef void (*f_outptr_putc)(Ctx *ctx, knh_outptr_t *, int ch);
-typedef size_t (*f_outptr_write)(Ctx *ctx, knh_outptr_t *, char *buf, size_t bufsiz);
-typedef void (*f_outptr_flush)(Ctx *ctx, knh_outptr_t *);
-typedef void (*f_outptr_close)(Ctx *ctx, knh_outptr_t *);
-
-typedef struct knh_outptr_apis_t {
-	int type; char *name;
-	f_outptr_open   fopen;
-	f_outptr_putc   fputc;
-	f_outptr_write  fwrite;
-	f_outptr_flush  fflush;
-	f_outptr_close  fclose;
-} knh_outptr_drvapi_t;
+/* @flag OutputStream.StoreBuffer OUT:3 (%s)->h.flag 'is:set:is:set' */
 
 typedef struct knh_OutputStream {
-	knh_outptr_t* outptr;
+	knh_io_t fd;
+	knh_iodrv_t driver;
 	knh_Bytes_t* ba;
-	knh_outptr_drvapi_t apis;
 	knh_String_t *enc;
 	struct knh_BytesConv_t* bconv;
 	knh_String_t*  urn;
@@ -721,21 +618,6 @@ typedef struct knh_OutputStream {
 	size_t indent;
 } knh_OutputStream_struct;
 
-///* ------------------------------------------------------------------------ */
-///* @ class Socket Object knh_ */
-//
-//typedef int knh_socket_t;
-//
-//typedef struct knh_Socket {
-//	knh_String_t* urn;
-//	int port;
-//	knh_socket_t sd;
-//	FILE *fp_in;
-//	FILE *fp_out;
-//	struct knh_InputStream_t  *in;
-//	struct knh_OutputStream_t *out;
-//} knh_Socket;
-
 /* ======================================================================== */
 
 typedef void   knh_connptr_t;
