@@ -286,8 +286,12 @@ KNHAPI(Exception*) new_Exception(Ctx *ctx, String *msg);
 KNHAPI(Exception*) new_Exception__b(Ctx *ctx, knh_bytes_t msg);
 KNHAPI(Exception*) new_Exception__errno(Ctx *ctx);
 KNHAPI(ClassSpec*) new_Unit(Ctx *ctx, char *tag, knh_float_t min, knh_float_t max, knh_float_t defv, knh_float_t step);
-KNHAPI(void) konoha_addInputStreamDriver(Ctx *ctx, char *alias, knh_inptr_drvapi_t *d);
-KNHAPI(void) konoha_addOutputStreamDriver(Ctx *ctx, char *alias, knh_outptr_drvapi_t *d);
+KNHAPI(InputStream*) new_FileInputStream(Ctx *ctx, knh_bytes_t file);
+KNHAPI(InputStream*) new_BytesInputStream(Ctx *ctx, Bytes *ba, size_t s, size_t e);
+KNHAPI(InputStream) *new_StringInputStream(Ctx *ctx, String *str, size_t s, size_t e);
+KNHAPI(void) konoha_addIODriver(Ctx *ctx, char *alias, knh_iodrv_t *d);
+KNHAPI(OutputStream*) new_FileOutputStream(Ctx *ctx, knh_bytes_t file, char *mode);
+KNHAPI(OutputStream*) new_BytesOutputStream(Ctx *ctx, Bytes *ba);
 KNHAPI(void) knh_putc(Ctx *ctx, OutputStream *w, int ch);
 KNHAPI(void) knh_write(Ctx *ctx, OutputStream *w, knh_bytes_t s);
 KNHAPI(void) knh_flush(Ctx *ctx, OutputStream *w);
