@@ -418,19 +418,19 @@ void knh_HashSet_add__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode);
 void knh_HashSet_remove__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode);
 void knh_HashSet_clear(Ctx *ctx, HashSet *o);
 /* ../class/konoha/knh_InputStream.c */
-void KNHINIT init_InputStream(Ctx *ctx);
 void knh_InputStream_struct_init(Ctx *ctx, knh_InputStream_struct *b, int init, Object *cs);
-void knh_InputStream_struct_traverse(Ctx *ctx, knh_InputStream_struct *b, f_traverse gc);
+void knh_InputStream_struct_traverse(Ctx *ctx, knh_InputStream_struct *b, f_traverse ftr);
 Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn);
-int knh_InputStream_getc(Ctx *ctx, InputStream *b);
-size_t knh_InputStream_read(Ctx *ctx, InputStream *b, char *buf, size_t bufsiz);
-String* knh_InputStream_readLine(Ctx *ctx, InputStream *b);
-void knh_InputStream_close(Ctx *ctx, InputStream *b);
-InputStream *new_InputStream__FILE(Ctx *ctx, FILE *fp);
-InputStream *new_InputStream__Bytes(Ctx *ctx, Bytes *ba, size_t s, size_t e);
+int knh_InputStream_getc(Ctx *ctx, InputStream *o);
+size_t knh_InputStream_read(Ctx *ctx, InputStream *o, char *buf, size_t bufsiz);
+String* knh_InputStream_readLine(Ctx *ctx, InputStream *o);
+void knh_InputStream_close(Ctx *ctx, InputStream *o);
+int knh_InputStream_isClosed(InputStream *o);
+InputStream *new_InputStream__stdio(Ctx *ctx, FILE *fp);
 /* ../class/konoha/knh_InputStream_api1.c */
 METHOD knh__InputStream_new(Ctx *ctx, knh_sfp_t *sfp);
 void knh_InputStream_setEncoding(Ctx *ctx, InputStream *o, String *enc);
+METHOD knh__InputStream_isClosed(Ctx *ctx, knh_sfp_t *sfp);
 void knh_InputStream__dump(Ctx *ctx, InputStream *b, OutputStream *w, String *m);
 Iterator* knh_InputStream_String__(Ctx *ctx, InputStream *o, Mapper *mpr);
 void knh_InputStream__k(Ctx *ctx, InputStream *o, OutputStream *w, String *m);
@@ -489,6 +489,11 @@ void knh_IntUnit_initId(Ctx *ctx, IntUnit *b, knh_flag_t flag, knh_class_t cid,
 void knh_IntUnit_initMinMax(Ctx *ctx, IntUnit *b, knh_integer_t min, knh_integer_t max);
 void knh_IntUnit_init_bfarray(Ctx *ctx, IntUnit *b, knh_int_t bf_start, knh_int_t bf_end);
 void knh_IntUnit_initDefaultValue(Ctx *ctx, IntUnit *b, knh_integer_t v);
+/* ../class/konoha/knh_iodrv.c */
+knh_iodrv_t *knh_System_getIODriver(Ctx *ctx, knh_bytes_t name);
+knh_iodrv_t knh_System_getDefaultIODriver();
+knh_iodrv_t knh_System_getStdIODriver();
+void KNHINIT init_IO(Ctx *ctx);
 /* ../class/konoha/knh_Iterator.c */
 void knh_Iterator_struct_init(Ctx *ctx, knh_Iterator_struct *b, int init, Object *cs);
 void knh_Iterator_struct_copy(Ctx *ctx, knh_Iterator_struct *b, knh_Iterator_struct *b2);
@@ -628,7 +633,6 @@ void knh_Object__empty(Ctx *ctx, Object *b, OutputStream *w, String *m);
 void knh_Object__refc(Ctx *ctx, Object *o, OutputStream *w, String *m);
 void knh_Object__addr(Ctx *ctx, Object *b, OutputStream *w, String *m);
 /* ../class/konoha/knh_OutputStream.c */
-void KNHINIT init_OutputStream(Ctx *ctx);
 void knh_OutputStream_struct_init(Ctx *ctx, knh_OutputStream_struct *b, int init, Object *cs);
 void knh_OutputStream_struct_traverse(Ctx *ctx, knh_OutputStream_struct *b, f_traverse gc);
 Object *knh_OutputStream_open(Ctx *ctx, OutputStream *o, String *urn, String *mode);
@@ -636,15 +640,16 @@ void knh_OutputStream_putc(Ctx *ctx, OutputStream *o, int ch);
 void knh_OutputStream_write(Ctx *ctx, OutputStream *o, knh_bytes_t buf);
 void knh_OutputStream_flush(Ctx *ctx, OutputStream *o);
 void knh_OutputStream_close(Ctx *ctx, OutputStream *o);
+int knh_OutputStream_isClosed(OutputStream *o);
 void knh_OutputStream_indent_inc(Ctx *ctx, OutputStream *o);
 void knh_OutputStream_indent_dec(Ctx *ctx, OutputStream *o);
 void knh_OutputStream_write_indent(Ctx *ctx, OutputStream *o);
 void knh_OutputStream_print_(Ctx *ctx, OutputStream *o, knh_bytes_t str, knh_bool_t isnl);
-OutputStream *new_OutputStream__FILE(Ctx *ctx, FILE *fp);
-OutputStream *new_OutputStream__Bytes(Ctx *ctx, Bytes *ba);
+OutputStream *new_OutputStream__stdio(Ctx *ctx, FILE *fp);
 /* ../class/konoha/knh_OutputStream_api1.c */
 METHOD knh__OutputStream_new(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__OutputStream_write(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__OutputStream_isClosed(Ctx *ctx, knh_sfp_t *sfp);
 void knh_OutputStream_setEncoding(Ctx *ctx, OutputStream *o, String *enc);
 METHOD knh__OutputStream_print(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__OutputStream_println(Ctx *ctx, knh_sfp_t *sfp);
