@@ -132,8 +132,8 @@ extern "C" {
 #define knh_Token_isSTR(tk_)            (SP(tk_)->tt == TT_STR)
 #define TT_TSTR                         ((knh_token_t)88)
 #define knh_Token_isTSTR(tk_)           (SP(tk_)->tt == TT_TSTR)
-#define TT_FMTSTR                       ((knh_token_t)89)
-#define knh_Token_isFMTSTR(tk_)         (SP(tk_)->tt == TT_FMTSTR)
+#define TT_EVAL                         ((knh_token_t)89)
+#define knh_Token_isEVAL(tk_)           (SP(tk_)->tt == TT_EVAL)
 #define TT_LABEL                        ((knh_token_t)90)
 #define knh_Token_isLABEL(tk_)          (SP(tk_)->tt == TT_LABEL)
 #define TT_METAN                        ((knh_token_t)91)
@@ -329,8 +329,7 @@ extern "C" {
 #define STT_OR                          ((knh_stmt_t)54)
 #define STT_PRINT                       ((knh_stmt_t)55)
 #define STT_ASSERT                      ((knh_stmt_t)56)
-#define STT_MAN                         ((knh_stmt_t)57)
-#define KNH_STMT_MAXSIZ                 ((knh_stmt_t)58)
+#define KNH_STMT_MAXSIZ                 ((knh_stmt_t)57)
 
 /* ------------------------------------------------------------------------ */
 
@@ -713,11 +712,6 @@ extern "C" {
 #define StmtASSERT_bool(stmt)           DP(stmt)->terms[0]
 #define ASSERT_bool                     0
 
-/* ======================================================================== */
-/* [man] */
-#define StmtMAN_keyword(stmt)           DP(stmt)->tokens[0]
-#define MAN_keyword                     0
-
 /* ======================================================================== */
 char* knh_token_tochar(knh_token_t t);
 knh_methodn_t knh_token_tomethodn(knh_token_t t);
