@@ -56,6 +56,12 @@ extern "C" {
 
 #define METHOD             void  KNH_CC_FASTCALL
 
+#ifdef KNH_USING_THREAD
+#define KNH_MT_VOLATILE           volatile
+#else
+#define KNH_MT_VOLATILE
+#endif
+
 #define KNH_USING_INT64 1
 
 /* ------------------------------------------------------------------------ */
@@ -202,8 +208,18 @@ typedef knh_ushort_t              knh_flag_t;    /* flag field */
 /* Long, knh_int64_t */
 /* ------------------------------------------------------------------------ */
 
+#ifndef LLONG_MIN
+#define LLONG_MIN -9223372036854775807LL
+#define LLONG_MAX  9223372036854775807LL
+#endif
+
+#ifndef ULLONG_MAX
+#define ULLONG_MAX 18446744073709551615ULL
+#endif
+
 typedef signed long long int           knh_int64_t;
 typedef unsigned long long int         knh_uint64_t;
+
 #define KNH_LONG_MAX   LLONG_MAX
 #define KNH_LONG_MIN   LLONG_MIN
 #define KNH_ULONG_MAX  ULLONG_MAX
@@ -256,11 +272,6 @@ typedef struct {
 	size_t     len;
 } knh_bytes_t;
 
-typedef struct {
-	knh_uchar_t   *mbuf;
-	size_t         mlen;
-} knh_mbytes_t;
-
 #define ismulti(c)             (((knh_uchar_t)c)>127)
 
 /* ------------------------------------------------------------------------ */
@@ -299,7 +310,11 @@ typedef struct knh_hObject_t {
 	knh_class_t bcid;
 	knh_class_t cid;
 #ifdef KNH_HOBJECT_REFC
-	volatile knh_uint_t refc;
+	KNH_MT_VOLATILE knh_uint_t refc;
+#endif
+#ifdef KNH_HOBJECT_ZCTFLAG
+	knh_Object_t *buf;
+	volatile knh_uint_t dirty;       /* Don't remove this until you draw something. */
 #endif
 } knh_hObject_t ;
 
@@ -330,9 +345,6 @@ typedef struct knh_Object_t {
 #define Any2            knh_Object_t
 #define UP(o)           (Object*)(o)
 
-//typedef void            Struct;
-
-//#define KNH_FIELDn(v,n)         ((Object**)(v))[n]
 #define KNH_FIELDn(v,n)            ((Script*)(v))->fields[(n)]
 
 /* ------------------------------------------------------------------------ */
@@ -511,9 +523,9 @@ typedef Object*    (*f_mapper)(Ctx *, Object *, struct knh_Mapper_t *);
 
 typedef knh_ushort_t             knh_line_t;
 
-/* ------------------------------------------------------------------------ */
+/* ======================================================================== */
 /* driver */
-/* ------------------------------------------------------------------------ */
+/* ======================================================================== */
 
 #define KNHINIT
 
@@ -524,8 +536,8 @@ typedef struct {
 
 #define KNH_DRVAPI_TYPE__UNKNOWN          0
 #define KNH_DRVAPI_TYPE__BCONV            1
-#define KNH_DRVAPI_TYPE__INPUTSTREAM      2
-#define KNH_DRVAPI_TYPE__OUTPUTSTREAM     3
+#define KNH_DRVAPI_TYPE__SCONV            2
+#define KNH_DRVAPI_TYPE__IO               3
 #define KNH_DRVAPI_TYPE__PARSER           4
 #define KNH_DRVAPI_TYPE__REGEX            5
 #define KNH_DRVAPI_TYPE__DB               6
@@ -533,8 +545,81 @@ typedef struct {
 #define IS_DRVAPI(c)   (0 < c && c < 7)
 
 /* ------------------------------------------------------------------------ */
-/* mutex */
+/* KNH_DRVAPI_TYPE__BYTECONV */
+
+struct knh_BytesConv_t;
+typedef size_t (*f_bconv)(Ctx *ctx, struct knh_BytesConv_t *o, knh_bytes_t t, struct knh_Bytes_t *ba);
+
+typedef struct {
+	int  type;
+	char *name;
+	f_bconv      fbconv;
+	f_bconv      fbconv_inverse;
+} knh_bconv_driapi_t;
+
 /* ------------------------------------------------------------------------ */
+/* KNH_DRVAPI_TYPE__IO */
+
+typedef knh_int_t knh_io_t;
+
+typedef knh_io_t (*f_io_open)(Ctx *ctx, Object *io, knh_bytes_t urn, char *mode);
+typedef size_t (*f_io_read)(Ctx *ctx, knh_io_t fd, char *buf, size_t bufsiz);
+typedef size_t (*f_io_write)(Ctx *ctx, knh_io_t fd, char *buf, size_t bufsiz);
+typedef void   (*f_io_close)(Ctx *ctx, knh_io_t fd);
+
+typedef struct {
+	int type; char *name;
+	size_t bufsiz;
+	f_io_open    fopen;
+	f_io_read    fread;
+	f_io_write   fwrite;
+	f_io_close   fclose;
+} knh_iodrv_t;
+
+/* ------------------------------------------------------------------------ */
+/* KNH_DRVAPI_TYPE__PARSER */
+
+typedef knh_Object_t* (*knh_fparser)(Ctx *, struct knh_String_t *p);
+
+typedef struct {
+	int   type;
+	char *name;
+	knh_type_t  rtype;
+	knh_fparser parser;
+} knh_parser_drvapi_t;
+
+/* ------------------------------------------------------------------------ */
+/* KNH_DRVAPI_TYPE__REGEX */
+
+typedef void knh_regex_t;
+
+#ifndef KNH_REGEX_NMATCH_MAX
+#define KNH_REGEX_NMATCH_MAX    16
+#endif
+
+typedef struct {
+	int rm_so;   /* start of match */
+	int rm_eo;   /* endo of match */
+	char *rm_name;  /* NULL, if not NAMED */
+} knh_regmatch_t;
+
+typedef knh_regex_t* (*knh_fregmalloc)(Ctx *);
+typedef int (*knh_fregcomp)(Ctx *, knh_regex_t *, char *pattern, int flags);
+typedef int (*knh_fregexec)(Ctx *, knh_regex_t *, char *str, size_t nmatch, knh_regmatch_t p[], int flags);
+typedef void (*knh_fregfree)(knh_regex_t *);
+
+typedef struct {
+	int  type;
+	char *name;
+	knh_fregmalloc regmalloc;
+	knh_fregcomp   regcomp;
+	knh_fregexec   regexec;
+	knh_fregfree   regfree;
+} knh_regex_drvapi_t;
+
+/* ======================================================================== */
+/* mutex */
+/* ======================================================================== */
 
 #ifdef KNH_USING_PTHREAD
 #define KNH_USING_THREAD 1
