@@ -66,8 +66,9 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 
 /* ======================================================================== */
 
-#define ARG_int(a)         ((a.i)->value)
-#define ARG_float(a)       ((a.f)->value)
+#define ARG_int(a)         ((int)(a.i)->value)
+#define ARG_uint(a)         ((unsigned int)(a.i)->value)
+#define ARG_float(a)       ((float)(a.f)->value)
 #define ARG_double(a)      ((double)(a.f)->value)
 #define ARG_char(a)        (knh_String_tochar(a.s))
 #define ARG_bytes(a)       (knh_Bytes_tobytes(a.s))
@@ -209,7 +210,7 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 #define VM_SCALL(ctx, n) { \
 		knh_sfp_t *mbp_ = ((Context*)ctx)->ebp - (n) ; \
 		KNH_ASSERT(IS_Method(mbp_[0].mtd)); \
-		KNH_ASSERT(IS_NOTNULL(mbp_[1].o)); \
+		KNH_ASSERT(IS_NOTNULL(mbp_[1].o) || knh_Method_isNullBase(mbp_[0].mtd)); \
 		mbp_[0].op = (knh_stackop_t)pc; \
 		(mbp_[0].mtd)->fcall_1(ctx, mbp_ + 1); \
 		mbp_[0].op = 0; \
@@ -230,7 +231,6 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 		((Context*)ctx)->ebp = mbp_; \
 	} \
 
-
 #define VM_CALL(ctx, n, mn) { \
 		knh_sfp_t *mbp_ = ctx->ebp - (n) ; \
 		KNH_SETv(ctx, mbp_[0].o, knh_Method_ufind(ctx, knh_Object_cid(mbp_[1].o), mn)); \
@@ -472,7 +472,7 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 		}else { \
 			knh_throwException(ctx, new_Exception__type(ctx, (e_), CLASS_Exception)); \
 		} \
-	} \
+	}
 
 /* ------------------------------------------------------------------------ */
 
@@ -483,6 +483,24 @@ int knh_Method_pctoline(Method *mtd, knhvmc_t *pc);
 
 #define VM_NOP(ctx)
 
+/* ------------------------------------------------------------------------ */
+/* ------------------------------------------------------------------------ */
+
+#define KNH_SECURE(ctx) \
+	if(((ctx)->flag & KNH_FLAG_CTXF_ADMIN) != KNH_FLAG_CTXF_ADMIN) { \
+		knh_throwException(ctx, new_Exception(ctx, TS_SecurityException)); \
+	} \
+
+#define KNH_BESTRICT(ctx, ff, cc) { \
+		if((ff cc) && ((ctx)->flag & KNH_FLAG_CTXF_STRICT) != KNH_FLAG_CTXF_STRICT) { \
+			knh_throwException(ctx, new_Exception__errno(ctx)); \
+		} \
+	}\
+
+
+/* ------------------------------------------------------------------------ */
+
+
 #ifdef __cplusplus
 }
 #endif
