@@ -76,6 +76,7 @@ static int knh_message_typedata[] = {
     /* KMSG_WARNING */ 0,
     /* KMSG_INFO */ 0,
     /* KMSG_ESYNTAX */ 1,
+    /* KMSG_EFUTURE */ 1,
     /* KMSG_EABORT */ 1,
     /* KMSG_EFORGET_PARENTHESIS */ 2,
     /* KMSG_EMISMATCH */ 1,
@@ -139,16 +140,14 @@ static int knh_message_typedata[] = {
     /* KMSG_ULABEL */ 1,
     /* KMSG_UOP */ 1,
     /* KMSG_UOP1 */ 1,
-    /* KMSG_UOP2 */ 2,
+    /* KMSG_UOP2 */ 1,
+    /* KMSG_WOPCMP */ 2,
     /* KMSG_UNEW */ 1,
     /* KMSG_UMAP */ 1,
     /* KMSG_INTOVERFLOW */ 2,
     /* KMSG_FLOATOVERFLOW */ 2,
-    /* KMSG_INT64OVERFLOW */ 2,
     /* KMSG_ETOOFEWPARAMS */ 1,
     /* KMSG_WTOOMANYPARAMS */ 2,
-    /* KMSG_EOP2 */ 1,
-    /* KMSG_EOPCMP */ 2,
     /* KMSG_ENULLPARAM */ 1,
     /* KMSG_WNULLPARAM */ 2,
     /* KMSG_ETYPEPARAM */ 1,
@@ -254,6 +253,13 @@ static char* knh_message_data[] = {
 #endif
         "ESYNTAX",
     
+    /* KMSG_EFUTURE */ 
+        /* en */ "Syntax Error. This will be supported in the near future.",
+#ifdef KNH_USING_I18N
+        /* ja */ "将来、サポートの予定です。今のところ、使用はご控えください",
+#endif
+        "EFUTURE",
+    
     /* KMSG_EABORT */ 
         /* en */ "Syntax Error. Otherwise, you'll find some compiler bugs.",
 #ifdef KNH_USING_I18N
@@ -702,6 +708,13 @@ static char* knh_message_data[] = {
 #endif
         "UOP2",
     
+    /* KMSG_WOPCMP */ 
+        /* en */ "Gosh!! Comparison of different types",
+#ifdef KNH_USING_I18N
+        /* ja */ "型が異なるため比較することはできません",
+#endif
+        "WOPCMP",
+    
     /* KMSG_UNEW */ 
         /* en */ "You have used an undefined constructor",
 #ifdef KNH_USING_I18N
@@ -730,13 +743,6 @@ static char* knh_message_data[] = {
 #endif
         "FLOATOVERFLOW",
     
-    /* KMSG_INT64OVERFLOW */ 
-        /* en */ "The given Int64 value was overflowed",
-#ifdef KNH_USING_I18N
-        /* ja */ "64ビット整数の最大値もしくは最小値を超えました",
-#endif
-        "INT64OVERFLOW",
-    
     /* KMSG_ETOOFEWPARAMS */ 
         /* en */ "This method needs more parameter(s)",
 #ifdef KNH_USING_I18N
@@ -751,20 +757,6 @@ static char* knh_message_data[] = {
 #endif
         "WTOOMANYPARAMS",
     
-    /* KMSG_EOP2 */ 
-        /* en */ "This operator is always binary",
-#ifdef KNH_USING_I18N
-        /* ja */ "この演算子は、2項演算子専用です",
-#endif
-        "EOP2",
-    
-    /* KMSG_EOPCMP */ 
-        /* en */ "Comparison of different types might lead to an unexpected confusion",
-#ifdef KNH_USING_I18N
-        /* ja */ "型が異なるため比較することはできません",
-#endif
-        "EOPCMP",
-    
     /* KMSG_ENULLPARAM */ 
         /* en */ "Null!! always occurs at the given argument",
 #ifdef KNH_USING_I18N
