@@ -127,7 +127,7 @@ static char* knh_token_array_tochar[] = {
     "NUM",  /* NUM */ 
     "STR",  /* STR */ 
     "TSTR",  /* TSTR */ 
-    "FMTSTR",  /* FMTSTR */ 
+    "EVAL",  /* EVAL */ 
     "LABEL",  /* LABEL */ 
     "METAN",  /* METAN */ 
     "PROPN",  /* PROPN */ 
@@ -251,7 +251,7 @@ static knh_methodn_t knh_token_array_tomethodn[] = {
     METHODN_NONAME,  /* NUM */ 
     METHODN_NONAME,  /* STR */ 
     METHODN_NONAME,  /* TSTR */ 
-    METHODN_NONAME,  /* FMTSTR */ 
+    METHODN_NONAME,  /* EVAL */ 
     METHODN_NONAME,  /* LABEL */ 
     METHODN_NONAME,  /* METAN */ 
     METHODN_NONAME,  /* PROPN */ 
@@ -375,7 +375,7 @@ static int knh_token_array_getOpPriority[] = {
     99,  /* NUM */ 
     99,  /* STR */ 
     99,  /* TSTR */ 
-    99,  /* FMTSTR */ 
+    99,  /* EVAL */ 
     0,  /* LABEL */ 
     0,  /* METAN */ 
     99,  /* PROPN */ 
@@ -499,7 +499,7 @@ static int knh_token_array_getOpSize[] = {
     0,  /* NUM */ 
     0,  /* STR */ 
     0,  /* TSTR */ 
-    0,  /* FMTSTR */ 
+    0,  /* EVAL */ 
     0,  /* LABEL */ 
     0,  /* METAN */ 
     0,  /* PROPN */ 
@@ -623,7 +623,7 @@ static knh_bool_t knh_token_array_isEndOfStmt[] = {
     0,  /* NUM */ 
     0,  /* STR */ 
     0,  /* TSTR */ 
-    0,  /* FMTSTR */ 
+    0,  /* EVAL */ 
     0,  /* LABEL */ 
     1,  /* METAN */ 
     0,  /* PROPN */ 
@@ -1039,7 +1039,6 @@ static char* knh_stmt_array_tochar[] = {
     "or",  /* or */ 
     "print",  /* print */ 
     "assert",  /* assert */ 
-    "man",  /* man */ 
     //(char*)0
 };
 
@@ -1109,7 +1108,6 @@ static size_t knh_stmt_array_size[] = {
     5,  /* or */ 
     5,  /* print */ 
     1,  /* assert */ 
-    1,  /* man */ 
     //(size_t)0
 };
 
@@ -1642,20 +1640,6 @@ static Stmt *new_StmtASSERT(Ctx *ctx, knh_tokens_t *tc)
 
 /* ------------------------------------------------------------------------ */
 
-/* ------------------------------------------------------------------------ */
-/* [man] */
-
-static Stmt *new_StmtMAN(Ctx *ctx, knh_tokens_t *tc)
-{
-	Stmt *o = new_Stmt(ctx, 0, STT_MAN);
-	knh_StmtMETA_add_prestmt(ctx, o, tc, -2);
-	knh_Stmt_add_ANY(ctx, o, tc); /* ANY */
-	knh_Stmt_add_SEMICOLON(ctx, o, tc); /* ; */
-	return o;
-}
-
-/* ------------------------------------------------------------------------ */
-
 /* ------------------------------------------------------------------------ */
 /* [stmt1] */
 
@@ -1776,8 +1760,8 @@ new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc)
         break;
     case TT_ALTLET:  /* ALTLET */
         break;
-    case TT_QUESTION:
-        return new_StmtMAN(ctx, tc);
+    case TT_QUESTION:  /* QUESTION */
+        break;
     case TT_SUBSETE:  /* SUBSETE */
         break;
     case TT_SUBSET:  /* SUBSET */
@@ -1867,7 +1851,7 @@ new_StmtSTMT1(Ctx *ctx, knh_tokens_t *tc)
     case TT_TSTR:  /* TSTR */
         tc->c -= 1;
         return new_StmtFUNCEXPR(ctx, tc);
-    case TT_FMTSTR:  /* FMTSTR */
+    case TT_EVAL:  /* EVAL */
         tc->c -= 1;
         return new_StmtFUNCEXPR(ctx, tc);
     case TT_LABEL: 
@@ -2013,7 +1997,6 @@ static int knh_stmt_array_isExpr[] = {
     1,  /* or */ 
     0,  /* print */ 
     0,  /* assert */ 
-    0,  /* man */ 
     //(int)0
 };
 
@@ -2083,7 +2066,6 @@ static f_visitdecl knh_stmt_array_decl[] = {
     knh_StmtDONE_decl,  /* or */ 
     knh_StmtDONE_decl,  /* print */ 
     knh_StmtDONE_decl,  /* assert */ 
-    knh_StmtDONE_decl,  /* man */ 
     //(f_visitdecl)0
 };
 
@@ -2153,7 +2135,6 @@ static f_visitname knh_stmt_array_name[] = {
     knh_StmtDONE_name,  /* or */ 
     knh_StmtDONE_name,  /* print */ 
     knh_StmtDONE_name,  /* assert */ 
-    knh_StmtDONE_name,  /* man */ 
     //(f_visitname)0
 };
 
@@ -2223,7 +2204,6 @@ static f_visitcmpl knh_stmt_array_cmpl[] = {
     knh_StmtOR_cmpl,  /* or */ 
     knh_StmtPRINT_cmpl,  /* print */ 
     knh_StmtDONE_cmpl,  /* assert */ 
-    knh_StmtDONE_cmpl,  /* man */ 
     //(f_visitcmpl)0
 };
 
