@@ -1570,47 +1570,47 @@ void KNH_ASM_JMP_IFF__OIDX(Ctx *ctx, Compiler *o, knh_bytes_t a1, Object* a2, kn
 
 /* ------------------------------------------------------------------------ */
 
-void KNH_ASM_JMP_IFTNN(Ctx *ctx, Compiler *o, knh_bytes_t a1, Term *v)
+void KNH_ASM_JMP_IFN(Ctx *ctx, Compiler *o, knh_bytes_t a1, Term *v)
 {
 	if(IS_Token((Token*)v)) {
 		Token *tk = (Token*)v;
-#ifdef ASMC_JMP_IFTNN__OBJ
+#ifdef ASMC_JMP_IFN__OBJ
 		if(SP(tk)->tt == TT_CONST || SP(tk)->tt == TT_MTDMPR) {
-			KNH_ASM_JMP_IFTNN__OBJ(ctx, o, a1, DP(tk)->data);
+			KNH_ASM_JMP_IFN__OBJ(ctx, o, a1, DP(tk)->data);
 			return;
 		}
-#endif/*ASMC_JMP_IFTNN__OBJ*/
-#ifdef ASMC_JMP_IFTNN__SFP
+#endif/*ASMC_JMP_IFN__OBJ*/
+#ifdef ASMC_JMP_IFN__SFP
 		if(SP(tk)->tt == TT_SFPIDX) {
-			KNH_ASM_JMP_IFTNN__sfp(ctx, o, a1, DP(tk)->index);
+			KNH_ASM_JMP_IFN__sfp(ctx, o, a1, DP(tk)->index);
 			return;
 		}
-#endif/*ASMC_JMP_IFTNN__SFP*/
-#ifdef ASMC_JMP_IFTNN__OFP
+#endif/*ASMC_JMP_IFN__SFP*/
+#ifdef ASMC_JMP_IFN__OFP
 		if(SP(tk)->tt == TT_FLDIDX) {
-			KNH_ASM_JMP_IFTNN__ofp(ctx, o, a1, DP(tk)->index);
+			KNH_ASM_JMP_IFN__ofp(ctx, o, a1, DP(tk)->index);
 			return;
 		}
-#endif/*ASMC_JMP_IFTNN__OFP*/
-#ifdef ASMC_JMP_IFTNN__OIDX
+#endif/*ASMC_JMP_IFN__OFP*/
+#ifdef ASMC_JMP_IFN__OIDX
 		if(SP(tk)->tt == TT_OBJIDX) {
-			KNH_ASM_JMP_IFTNN__OIDX(ctx, o, a1, DP(tk)->data, DP(tk)->index);
+			KNH_ASM_JMP_IFN__OIDX(ctx, o, a1, DP(tk)->data, DP(tk)->index);
 			return;
 		}
-#endif/*ASMC_JMP_IFTNN__OIDX*/
-#ifdef ASMC_JMP_IFTNN__EBP
+#endif/*ASMC_JMP_IFN__OIDX*/
+#ifdef ASMC_JMP_IFN__EBP
 		if(SP(tk)->tt == TT_EBPIDX) {
-			KNH_ASM_JMP_IFTNN__ebp(ctx, o, a1, DP(tk)->index);
+			KNH_ASM_JMP_IFN__ebp(ctx, o, a1, DP(tk)->index);
 			return;
 		}
 		if(SP(tk)->tt == TT_ASIS) {
-			KNH_ASM_JMP_IFTNN__ebp(ctx, o, a1, 0);
+			KNH_ASM_JMP_IFN__ebp(ctx, o, a1, 0);
 			return;
 		}
-#endif/*ASMC_JMP_IFTNN__EBP*/
+#endif/*ASMC_JMP_IFN__EBP*/
 		if(SP(tk)->tt == TT_SYSCONST) {
 			KNH_ASM_PUSH_SYSCONST(ctx, o, DP(tk)->index);
-			KNH_ASM_JMP_IFTNN__ebp(ctx, o, a1, 0);
+			KNH_ASM_JMP_IFN__ebp(ctx, o, a1, 0);
 			return;
 		}
 		if(SP(tk)->tt == TT_ERR) {
@@ -1624,23 +1624,23 @@ void KNH_ASM_JMP_IFTNN(Ctx *ctx, Compiler *o, knh_bytes_t a1, Term *v)
 	}else {
 		KNH_ASSERT(IS_Stmt((Stmt*)v));
 
-#ifdef ASMC_JMP_IFTNN__EBP
-		KNH_ASM_JMP_IFTNN__ebp(ctx, o, a1, 0);
+#ifdef ASMC_JMP_IFN__EBP
+		KNH_ASM_JMP_IFN__ebp(ctx, o, a1, 0);
 		return;
 #else
 		DBG_P("unsupported stmt stt=%s", knh_stmt_tochar(DP((Stmt*)v)->stt));
 		KNH_ASSERT(ctx == NULL); /* stop here!!*/
-#endif/*ASMC_JMP_IFTNN__EBP*/
+#endif/*ASMC_JMP_IFN__EBP*/
 	}
 }
 
 /* ------------------------------------------------------------------------ */
 
-void KNH_ASM_JMP_IFTNN__ebp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ebp_t a2)
+void KNH_ASM_JMP_IFN__ebp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ebp_t a2)
 {
 	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iftnn_ebp_t *bw = (knh_asmc_jmp_iftnn_ebp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFTNN_ebp_SIZ);
-		bw->code = ASMC_JMP_IFTNN__EBP;
+		knh_asmc_jmp_ifn_ebp_t *bw = (knh_asmc_jmp_ifn_ebp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFN_ebp_SIZ);
+		bw->code = ASMC_JMP_IFN__EBP;
 		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
 		bw->a2 = a2;
 	}
@@ -1648,11 +1648,11 @@ void KNH_ASM_JMP_IFTNN__ebp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ebp_t a
 
 /* ------------------------------------------------------------------------ */
 
-void KNH_ASM_JMP_IFTNN__sfp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_sfp_t a2)
+void KNH_ASM_JMP_IFN__sfp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_sfp_t a2)
 {
 	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iftnn_sfp_t *bw = (knh_asmc_jmp_iftnn_sfp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFTNN_sfp_SIZ);
-		bw->code = ASMC_JMP_IFTNN__SFP;
+		knh_asmc_jmp_ifn_sfp_t *bw = (knh_asmc_jmp_ifn_sfp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFN_sfp_SIZ);
+		bw->code = ASMC_JMP_IFN__SFP;
 		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
 		bw->a2 = a2;
 	}
@@ -1660,11 +1660,11 @@ void KNH_ASM_JMP_IFTNN__sfp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_sfp_t a
 
 /* ------------------------------------------------------------------------ */
 
-void KNH_ASM_JMP_IFTNN__ofp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ofp_t a2)
+void KNH_ASM_JMP_IFN__ofp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ofp_t a2)
 {
 	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iftnn_ofp_t *bw = (knh_asmc_jmp_iftnn_ofp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFTNN_ofp_SIZ);
-		bw->code = ASMC_JMP_IFTNN__OFP;
+		knh_asmc_jmp_ifn_ofp_t *bw = (knh_asmc_jmp_ifn_ofp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFN_ofp_SIZ);
+		bw->code = ASMC_JMP_IFN__OFP;
 		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
 		bw->a2 = a2;
 	}
@@ -1672,127 +1672,11 @@ void KNH_ASM_JMP_IFTNN__ofp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ofp_t a
 
 /* ------------------------------------------------------------------------ */
 
-void KNH_ASM_JMP_IFTNN__OIDX(Ctx *ctx, Compiler *o, knh_bytes_t a1, Object* a2, knhvm_u1_t a3)
+void KNH_ASM_JMP_IFN__OIDX(Ctx *ctx, Compiler *o, knh_bytes_t a1, Object* a2, knhvm_u1_t a3)
 {
 	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iftnn_oidx_t *bw = (knh_asmc_jmp_iftnn_oidx_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFTNN_OIDX_SIZ);
-		bw->code = ASMC_JMP_IFTNN__OIDX;
-		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
-		bw->a2 = a2;
-		knh_Object_RCinc(bw->a2);
-		bw->a3 = a3;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-void KNH_ASM_JMP_IFFN(Ctx *ctx, Compiler *o, knh_bytes_t a1, Term *v)
-{
-	if(IS_Token((Token*)v)) {
-		Token *tk = (Token*)v;
-#ifdef ASMC_JMP_IFFN__OBJ
-		if(SP(tk)->tt == TT_CONST || SP(tk)->tt == TT_MTDMPR) {
-			KNH_ASM_JMP_IFFN__OBJ(ctx, o, a1, DP(tk)->data);
-			return;
-		}
-#endif/*ASMC_JMP_IFFN__OBJ*/
-#ifdef ASMC_JMP_IFFN__SFP
-		if(SP(tk)->tt == TT_SFPIDX) {
-			KNH_ASM_JMP_IFFN__sfp(ctx, o, a1, DP(tk)->index);
-			return;
-		}
-#endif/*ASMC_JMP_IFFN__SFP*/
-#ifdef ASMC_JMP_IFFN__OFP
-		if(SP(tk)->tt == TT_FLDIDX) {
-			KNH_ASM_JMP_IFFN__ofp(ctx, o, a1, DP(tk)->index);
-			return;
-		}
-#endif/*ASMC_JMP_IFFN__OFP*/
-#ifdef ASMC_JMP_IFFN__OIDX
-		if(SP(tk)->tt == TT_OBJIDX) {
-			KNH_ASM_JMP_IFFN__OIDX(ctx, o, a1, DP(tk)->data, DP(tk)->index);
-			return;
-		}
-#endif/*ASMC_JMP_IFFN__OIDX*/
-#ifdef ASMC_JMP_IFFN__EBP
-		if(SP(tk)->tt == TT_EBPIDX) {
-			KNH_ASM_JMP_IFFN__ebp(ctx, o, a1, DP(tk)->index);
-			return;
-		}
-		if(SP(tk)->tt == TT_ASIS) {
-			KNH_ASM_JMP_IFFN__ebp(ctx, o, a1, 0);
-			return;
-		}
-#endif/*ASMC_JMP_IFFN__EBP*/
-		if(SP(tk)->tt == TT_SYSCONST) {
-			KNH_ASM_PUSH_SYSCONST(ctx, o, DP(tk)->index);
-			KNH_ASM_JMP_IFFN__ebp(ctx, o, a1, 0);
-			return;
-		}
-		if(SP(tk)->tt == TT_ERR) {
-			KNH_ASSERT(IS_bString(DP(tk)->data));
-			KNH_ASM_THROW__OBJ(ctx, o, DP(tk)->data);
-			knh_Compiler_setStopped(o, 1);
-			return;
-		}
-		DBG_P("unsupported token tt=%s", knh_token_tochar(SP(tk)->tt));
-		KNH_ASSERT(ctx == NULL); /* stop here!!*/
-	}else {
-		KNH_ASSERT(IS_Stmt((Stmt*)v));
-
-#ifdef ASMC_JMP_IFFN__EBP
-		KNH_ASM_JMP_IFFN__ebp(ctx, o, a1, 0);
-		return;
-#else
-		DBG_P("unsupported stmt stt=%s", knh_stmt_tochar(DP((Stmt*)v)->stt));
-		KNH_ASSERT(ctx == NULL); /* stop here!!*/
-#endif/*ASMC_JMP_IFFN__EBP*/
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-void KNH_ASM_JMP_IFFN__ebp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ebp_t a2)
-{
-	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iffn_ebp_t *bw = (knh_asmc_jmp_iffn_ebp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFFN_ebp_SIZ);
-		bw->code = ASMC_JMP_IFFN__EBP;
-		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
-		bw->a2 = a2;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-void KNH_ASM_JMP_IFFN__sfp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_sfp_t a2)
-{
-	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iffn_sfp_t *bw = (knh_asmc_jmp_iffn_sfp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFFN_sfp_SIZ);
-		bw->code = ASMC_JMP_IFFN__SFP;
-		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
-		bw->a2 = a2;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-void KNH_ASM_JMP_IFFN__ofp(Ctx *ctx, Compiler *o, knh_bytes_t a1, knhvm_ofp_t a2)
-{
-	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iffn_ofp_t *bw = (knh_asmc_jmp_iffn_ofp_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFFN_ofp_SIZ);
-		bw->code = ASMC_JMP_IFFN__OFP;
-		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
-		bw->a2 = a2;
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-void KNH_ASM_JMP_IFFN__OIDX(Ctx *ctx, Compiler *o, knh_bytes_t a1, Object* a2, knhvm_u1_t a3)
-{
-	if(!knh_Compiler_isStopped(o)) {
-		knh_asmc_jmp_iffn_oidx_t *bw = (knh_asmc_jmp_iffn_oidx_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFFN_OIDX_SIZ);
-		bw->code = ASMC_JMP_IFFN__OIDX;
+		knh_asmc_jmp_ifn_oidx_t *bw = (knh_asmc_jmp_ifn_oidx_t*)knh_Compiler_asmmalloc(ctx, o, KNH_ASMC_JMP_IFN_OIDX_SIZ);
+		bw->code = ASMC_JMP_IFN__OIDX;
 		bw->a1 = knh_Compiler_labelId(ctx, o, a1);
 		bw->a2 = a2;
 		knh_Object_RCinc(bw->a2);
