@@ -896,9 +896,9 @@ knh_System_t *new_System0(void)
 	knh_System_t *o = (knh_System_t*)new_ObjectX_malloc(NULL, FLAG_System, CLASS_Object, CLASS_System, sizeof(knh_System_struct));
 	KNH_INITv(DP(o)->ns,    new_NameSpace(NULL, (NameSpace*)KNH_NULL, TS_EMPTY));
 	KNH_INITv(DP(o)->enc,   new_String__T(NULL, konoha_encoding()));
-	KNH_INITv(DP(o)->in,    new_InputStream__FILE(NULL, stdin));
-	KNH_INITv(DP(o)->out,   new_OutputStream__FILE(NULL, stdout));
-	KNH_INITv(DP(o)->err,   new_OutputStream__FILE(NULL, stderr));
+	KNH_INITv(DP(o)->in,    new_InputStream__stdio(NULL, stdin));
+	KNH_INITv(DP(o)->out,   new_OutputStream__stdio(NULL, stdout));
+	KNH_INITv(DP(o)->err,   new_OutputStream__stdio(NULL, stderr));
 	knh_InputStream_setEncoding(NULL, DP(o)->in,   DP(o)->enc);
 	knh_OutputStream_setEncoding(NULL, DP(o)->out, DP(o)->enc);
 	knh_OutputStream_setEncoding(NULL, DP(o)->err, DP(o)->enc);
