@@ -28,6 +28,7 @@
 /* ************************************************************************ */
 
 #include<konoha.h>
+#define KNH_PKGNAME "demo"
 #
 #ifdef __cplusplus
 extern "C" {
@@ -39,91 +40,6 @@ extern "C" {
 
 /* ------------------------------------------------------------------------ */
 
-//knh_class_t knh_system_loadEnum(Ctx *ctx, knh_bytes_t urn)
-//{
-//	knh_index_t idx = knh_bytes_rindex(urn, '/');
-//	DEBUG_ASSERT(idx != -1);
-//	int ch = urn.buf[idx+1];
-//	knh_class_t cid/*, tcid*/;
-//
-//	switch(ch) {
-//	case 'A':
-//		if(ISB(urn, "http://enum/Age")) {
-//			cid = KNH_TENUM(ctx, "http://enum/Age", "", 0, 200, 0, 0, 120);
-//			return cid;
-//		}
-//		break;
-//
-//	case 'M':
-//		if(ISB(urn, "http://enum/Month")) {
-//			cid = KNH_TENUM(ctx, "http://enum/Month", "", 1, 12, 1, 1, 12);
-//			return cid;
-//		}
-//		break;
-//
-//	}
-//
-//	return CLASS_Int;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//knh_class_t knh_system_loadUnit(Ctx *ctx, knh_bytes_t urn)
-//{
-//	knh_index_t idx = knh_bytes_rindex(urn, '/');
-//	DEBUG_ASSERT(idx != -1);
-//	int ch = urn.buf[idx+1];
-//	knh_class_t cid, tcid;
-//
-//	switch(ch) {
-//	case 'C':
-//		if(ISB(urn, "http://unit/Celsius")) {
-//			cid = KNH_TUNIT(ctx, "http://unit/Celsius", "C", -273.15, KNH_FLOAT_MAX, 0.0, 0.01);
-//			return cid;
-//		}
-//		break;
-//
-//	case 'F':
-//		if(ISB(urn, "http://unit/Fahrenheit")) {
-//			cid = KNH_TUNIT(ctx, "http://unit/Fahrenheit", "F", -459.67, KNH_FLOAT_MAX, 32.0, 0.01);
-//			tcid = knh_tclass_classURN(ctx, CLASS_Float, B("http://unit/Celsius"));
-//			KNH_TAFFINE(ctx, cid, tcid, 5.0/9.0, -32.0*5.0/9.0);
-//			return cid;
-//		}
-//		break;
-//	}
-//
-//	return CLASS_Float;
-//}
-//
-///* ------------------------------------------------------------------------ */
-//
-//knh_class_t knh_system_loadVocabulary(Ctx *ctx, knh_bytes_t urn)
-//{
-////	knh_index_t idx = knh_bytes_rindex(urn, '/');
-////	DEBUG_ASSERT(idx != -1);
-////	int ch = urn.buf[idx+1];
-////	knh_class_t cid, tcid;
-////
-////	switch(ch) {
-////	case 'S':
-////		if(ISB(urn, "http://konoha/EnglishSeason")) {
-//// 			cid = KNH_TUNIT(ctx, "http://unit/Fahrenheit", "F", -459.67, KNH_FLOAT_MAX, 32.0, 0.01);
-////			tcid = knh_tclass_classURN(ctx, CLASS_Float, B("http://unit/Celsius"));
-////			KNH_TAFFINE(ctx, cid, tcid, 5.0/9.0, -32.0*5.0/9.0);
-////			return cid;
-////		}
-////		break;
-////
-////	}
-////
-////
-//	return CLASS_String;
-//}
-//
-//
-//
-
 static
 ClassSpec *makespec_Celsius(Ctx *ctx, knh_class_t bcid, char *extra)
 {
@@ -186,16 +102,7 @@ ClassSpec *makespec_ItalianCity__it(Ctx *ctx, knh_class_t bcid, char *extra)
 /* ======================================================================== */
 /* [KNHAPI] */
 
-KNHAPI(void) knhdemo_traverse(Ctx *ctx, f_traverse ftr)
-{
-	if(konoha_issweep(ftr)) {
-		//KNH_NOTICE(ctx, "loading math..");
-	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-KNHAPI(int)  knhdemo_init(Ctx *ctx)
+int knhdemo_init(Ctx *ctx)
 {
 	KNH_NOTICE(ctx, "loading ai2008..");
 	konoha_addClassSpecFunc(ctx, "http://konoha/ItalianCity.en", makespec_ItalianCity__en);
@@ -212,6 +119,34 @@ KNHAPI(int)  knhdemo_init(Ctx *ctx)
 
 /* ------------------------------------------------------------------------ */
 
+void knhdemo_traverse(Ctx *ctx, f_traverse ftr)
+{
+	if(konoha_issweep(ftr)) {
+		//KNH_NOTICE(ctx, "loading math..");
+	}
+}
+
+/* ------------------------------------------------------------------------ */
+
+static knh_pkgmeta_t PKGDATA = {
+	KONOHA_BUILDID, /* Don't change this */
+	KNH_PKGNAME,    /* package name (not NULL) */
+	"1.0",  /* package version (not NULL) */
+	NULL,   /* additional information */
+	"http://www.ubicg.ynu.ac.jp/", /* URL */
+	knhdemo_init,
+	knhdemo_traverse,
+	NULL
+};
+
+KNHAPI(knh_pkgmeta_t*) knhdemo_load(Ctx *ctx)
+{
+	return &(PKGDATA);
+}
+
+/* ------------------------------------------------------------------------ */
+
+
 #ifdef __cplusplus
 }
 #endif
